import asyncio
import binascii
import configparser
import ipaddress
import json
import logging
import os
import threading
import csv
import platform as pt
import time
import subprocess
from pprint import pprint

import platformdirs
import yaml
import getmac
import platform
import socket
from pathlib import Path
import click
import keyring
import requests
from environs import Env
from keyring.errors import NoKeyringError
from pysnmp.hlapi import *
from sqlitedict import SqliteDict
from semver.version import Version as sem_version
from packaging import version as pkg_version
# import pandas as pd
import re

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

def extract_info_linux(package_string):
    # Original regular expression pattern
    original_pattern = r'^(.*?)_([^_]+)_([^\s]+)$'

    # Additional pattern to handle strings like "whiptail-0.52.20-1ubuntu1"
    additional_pattern = r'^([^-\s]+)-([\d.]+)-(\S+)$'

    # Pattern for strings like "ncurses-6.1-10.20180224.el8"
    new_pattern = r'^([^-\s]+)-([\d.]+)-([\d.]+)\.([^\s]+)$'

    # Pattern for strings like "ncurses-6.1-10.20180224.el8"
    second_pattern = r'(.*)-(.*)-(.*).(.*)'
    third_pattern = r'\d+(\.\d+)*'

    # Try the original pattern first
    match = re.match(original_pattern, package_string)

    if match:
        name = match.group(1)
        version = match.group(2)
        architecture = match.group(3)
        return {"name": name, "version": version, "arch": architecture}
    else:
        # If the original pattern didn't match, try the additional pattern
        match = re.match(additional_pattern, package_string)

        if match:
            name = match.group(1)
            version = match.group(2)
            architecture = match.group(3)
            return {"name": name, "version": version, "arch": architecture}
        else:
            # If the additional pattern didn't match, try the new pattern
            match = re.match(new_pattern, package_string)

            if match:
                name = match.group(1)
                version = match.group(2)
                release = match.group(3)
                architecture = match.group(4)
                return {"name": name, "version": version, "arch": architecture}
            else:
                # If the new pattern didn't match, try the second pattern
                match = re.match(second_pattern, package_string)

                if match:
                    name = match.group(1)
                    version = match.group(2)
                    release = match.group(3)
                    architecture = match.group(4)
                    return {"name": name, "version": version, "arch": architecture}
                else:
                    return None


def extract_info_windows(package_string):
    try:
        # Essayer de décoder la chaîne hexadécimale
        decoded_string = binascii.unhexlify(package_string[2:]).decode('latin-1')

        # Utilisation d'une expression régulière pour extraire le nom, la version et l'architecture sous Windows
        pattern = r'^(.*?)\s*-\s*(.*?)\s*\((.*?)\)$'
        match = re.match(pattern, decoded_string)

        if match:
            name = match.group(1)
            version = match.group(2)
            architecture = match.group(3)
            return {"name": name, "version": version, "architecture": architecture}
        else:
            return {"name": decoded_string, "version": None, "architecture": None}
    except (binascii.Error, ValueError):
        # La chaîne n'est pas hexadécimale, traiter comme une chaîne normale
        pattern = r'^(.*?)\s*-\s*(.*?)\s*\((.*?)\)$'
        match = re.match(pattern, package_string)

        if match:
            name = match.group(1)
            version = match.group(2)
            architecture = match.group(3)
            return {"name": name, "version": version, "architecture": architecture}
        else:
            return {"name": package_string, "version": None, "architecture": None}


def extract_windows_host_info(input_str):
    # Expression régulière pour extraire l'architecture matérielle (premier mot après "Hardware:")
    hardware_architecture_pattern = r'Hardware:\s(\S+)'

    # Expression régulière pour extraire les informations logicielles
    software_pattern = r'Software:\s(.*?)\sVersion\s(\d+\.\d+).*\(Build\s(\d+)\s.*\)$'

    # Rechercher les correspondances dans la chaîne d'entrée
    hardware_architecture_match = re.search(hardware_architecture_pattern, input_str, re.IGNORECASE)
    software_match = re.search(software_pattern, input_str)

    # Extraire l'architecture matérielle
    hardware_architecture = hardware_architecture_match.group(1).strip() if hardware_architecture_match else None

    # Extraire les informations logicielles
    software_name = software_match.group(1).strip() if software_match else None
    software_version = software_match.group(2).strip() if software_match else None
    software_build = software_match.group(3).strip() if software_match else None

    return {
        "arch": hardware_architecture,
        "os_name": software_name,
        "kernel_version": software_version,
        "build": software_build
    }


def extract_info_macos(package_string):
    # Utilisation d'une expression régulière pour extraire le nom et la version sous macOS
    pattern = r'^(.*?)\s+(\S+)\s*$'
    match = re.match(pattern, package_string)

    if match:
        name = match.group(1)
        version = match.group(2)
        if '(null)' in version:
            version = None
        if '(null)' in name:
            name = None
        return {"name": name, "version": version, "arch": None}
    else:
        return None


linux_version_pattern = re.compile(
    r"^"
    # epoch must start with a digit
    r"(\d+:)?"
    # upstream must start with a digit
    r"\d"
    r"("
    # upstream  can contain only alphanumerics and the characters . + -
    # ~ (full stop, plus, hyphen, tilde)
    r"[A-Za-z0-9\.\+\~\-]+"
    r"|"
    # If there is no debian_revision then hyphens are not allowed in version.
    r"[A-Za-z0-9\.\+\~]+-[A-Za-z0-9\+\.\~]+"
    r")?"
    r"$"
)
irregular_version_pattern = re.compile(r'\d+(\.\d+)*')

def parse_version(text):
    """ Semantic Versioning (SemVer)
     Date-based Versioning
     Alphanumeric or Custom Schemes
     Debian based version parser
     Ubuntu based version parser
     parse version with build:
    """
    if not text:
        return None

    if linux_version_pattern.match(text):
        match = linux_version_pattern.search(text)
        if match:
            version = match.group()
            if ":" in version:
                epoch, _, version = version.partition(":")
                epoch = int(epoch)
            else:
                epoch = 0

            if "-" in version:
                upstream, _, revision = version.rpartition("-")
            else:
                upstream = version
                revision = "0"

            version = upstream
            regex_matched = False

            if 'ubuntu' in version:
                match = irregular_version_pattern.search(version)
                if match:
                    regex_matched = True
                    version = match.group()
            elif 'debian' in version:
                match = irregular_version_pattern.search(version)
                if match:
                    regex_matched = True
                    version = match.group()
            elif 'git' in version:
                match = irregular_version_pattern.search(version)
                if match:
                    regex_matched = True
                    version = match.group()
            elif '-' in version:
                match = irregular_version_pattern.search(version)
                if match:
                    regex_matched = True
                    version = match.group()
            else:
                match = irregular_version_pattern.search(version)
                if match:
                    regex_matched = True
                    version = match.group()

            parsed = None
            if not regex_matched:
                try:
                    parsed = sem_version.parse(version)
                except ValueError:
                    try:
                        parsed = pkg_version.parse(version)
                    except pkg_version.InvalidVersion:
                        parsed = None

            if parsed:
                parsed_split_len = len(str(parsed).split("."))
                if parsed_split_len < 3:
                    version = [str(parsed.major), str(parsed.minor)]
                elif parsed_split_len == 3:
                    try:
                        version = [str(parsed.major), str(parsed.minor), str(parsed.patch)]
                    except AttributeError:
                        version = [str(parsed.major), str(parsed.minor), str(parsed.micro)]
                else:
                    version = parsed

                if isinstance(version, list):
                    version = ".".join(version)
                else:
                    version = version
            else:
                if not regex_matched:
                    print(f'Cannot definitely parse version {text}')
            return version


def snmp_scanner(ip, ports: list = None):
    if ports is None:
        ports = [161]

    open_ports = []

    for port in ports:
        try:
            # Créez un objet socket
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

            # Fixez un timeout court pour la connexion
            s.settimeout(1)

            # Tentez de se connecter à l'adresse IP et au port donnés
            s.connect((ip, port))
            # Si la connexion réussit, le port est ouvert
            open_ports.append(port)
        except Exception as e:
            print(f'Connexion exception the host must probably filtering the port. Reason: {e}')
    return open_ports



async def get_packages_async(hostname, community, os_name):
    var_bind = '1.3.6.1.2.1.25.6.3.1.2'

    def parse_version_append(ver, res, host):
        ver = parse_version(ver)
        d = {
            "name": pkg_info['name'].replace('"', ",") if '"' in pkg_info['name'] else pkg_info['name'],
            "version": ver,
        }
        res.append(d)

    iterator = await snmp_query_v2(var_bind=var_bind, hostname=hostname, community=community)
    
    result = []
    parsed_values = await parse_snmp_response(iterator, var_bind)
    threads = []
    extract_fnc = extract_info_windows
    if os_name == "linux":
        extract_fnc = extract_info_linux
    else:
        extract_fnc = extract_info_macos

    for value in parsed_values:
        pretty_value = value.prettyPrint()
        pkg_info = extract_fnc(pretty_value)
        if pkg_info:
            thread = threading.Thread(target=parse_version_append,
                                      args=(pkg_info['version'], result, hostname))
            thread.start()
            threads.append(thread)

    for thread in threads:
        thread.join()
    return result


async def get_host_info_async(hostname, community):
    var_binds = {
        'sys_hostname_bind': '1.3.6.1.2.1.1.5.0',
        'sys_descr_bind': '1.3.6.1.2.1.1.1.0',
    }
    result = {}
    for var_bind_name, var_bind_value in var_binds.items():
        iterator = await snmp_get_query_v2(var_bind=var_bind_value, hostname=hostname, community=community)
        parsed_values = await parse_snmp_response(iterator, var_bind_value)
        
        for value in parsed_values:
            value = value.prettyPrint()
            print(  f" host info {value}")
            lower_value = value.lower()
            if var_bind_name == 'sys_hostname_bind':
                result["host_name"] = value
                result["mac"] = getmac.get_mac_address(ip=hostname)
            elif var_bind_name == 'sys_descr_bind':
                if "windows" in lower_value:
                    info = extract_windows_host_info(value)
                    result["os_name"] = info.get("os_name")
                    result["kernel_version"] = info.get("kernel_version")
                    result["arch"] = info.get("arch")
                    result["build"] = info.get("build")
                elif "darwin" in lower_value:
                    partitioned = value.split()
                    if partitioned:
                        result["os_name"] = partitioned[0]
                        result["kernel_version"] = partitioned[2]
                        result["arch"] = partitioned[-1]
                elif "linux" in lower_value:
                    partitioned = value.split()
                    if partitioned:
                        result["os_name"] = partitioned[0]
                        result["kernel_version"] = partitioned[2]
                        result["arch"] = partitioned[-1]
                else:
                    print("Cannot parse unix based host informations.")
    return result


async def parse_snmp_response(iterator, var_bind):
    def sub_thread_iter(bind, var, res, stop):
        oid, value = bind
        if var not in str(oid):
            stop.set()  # Utilisez .set() pour définir la variable de contrôle à True
        else:
            res.append(value)

    def threaded_iterator(indication, status, binds, stop, res, var_b):
        if indication:
            return
        elif status:
            return
        else:
            sub_threads = []
            for bind in binds:
                th = threading.Thread(target=sub_thread_iter, args=(bind, var_b, res, stop))
                th.start()
                if stop.is_set():  # Utilisez .is_set() pour vérifier la variable de contrôle
                    break
                sub_threads.append(th)

            for th in sub_threads:
                th.join()

    result = []
    stop_loop = threading.Event()  # Utilisez un objet Event pour la variable de contrôle
    threads = []
    for error_indication, error_status, error_index, var_binds in iterator:
        thread = threading.Thread(target=threaded_iterator,
                                  args=(error_indication, error_status, var_binds, stop_loop, result, var_bind))
        thread.start()
        if stop_loop.is_set():  # Utilisez .is_set() pour vérifier la variable de contrôle
            break
        threads.append(thread)

    for thread in threads:
        thread.join()
    return result


async def snmp_query_v2(var_bind, hostname, community="public"):
    snmp_engine = SnmpEngine()
    return nextCmd(
        snmp_engine,
        CommunityData(community),
        UdpTransportTarget((hostname, 161), timeout=2.0, retries=0),
        ContextData(),
        ObjectType(ObjectIdentity(var_bind)),
    )


async def snmp_get_query_v2(var_bind, hostname, community="public"):
    snmp_engine = SnmpEngine()
    try:
        return getCmd(
            snmp_engine,
            CommunityData(community),
            UdpTransportTarget((hostname, 161), timeout=2.0, retries=0),
            ContextData(),
            ObjectType(ObjectIdentity(var_bind)),
        )
    except:
        return None


async def snmp_query_v3(var_bind, hostname, username, auth_key, priv_key, auth_protocol=usmHMACSHAAuthProtocol,
                        priv_protocol=usmAesCfb128Protocol):
    snmp_engine = SnmpEngine()
    try:
        return nextCmd(
            snmp_engine,
            UsmUserData(username, auth_key, priv_key, auth_protocol, priv_protocol),
            UdpTransportTarget(hostname),
            ContextData(),
            ObjectType(ObjectIdentity(var_bind)),
        )
    except:
        return None


def scan_up_host_and_append(ip, active_hosts):
    active = is_ip_active(ip=ip, all_active=True)
    if active:
        active_hosts.add(ip)
    return active_hosts


def get_snmp_hosts(network, use_range, range_start, range_end):
    config = Configuration.load(config_file_path=config_file)
    active_hosts = set()
    cidr = config.get_value('network', 'cidr', default=24)
    exempt = config.get_value('network', 'exempt')
    snmp_port = config.get_value('network', 'snmp', 'port', default=161)

    if not snmp_port:
        raise ValueError("The configured snmp port must be provided.")

    if network and use_range:
        if not range_start or not range_end:
            raise ValueError("The network range ip address must be provided.")
        hosts = get_possible_active_hosts_by_range(range_start, range_end, exempt=exempt)
    else:
        hosts = get_possible_active_hosts(ip_address=network, cidr=cidr, exempt=exempt)

    threads = []
    for host in hosts:
        thread = threading.Thread(target=scan_snmp_and_append, args=(host, snmp_port, active_hosts))
        thread.start()
        threads.append(thread)

    for thread in threads:
        thread.join()
    return active_hosts


async def handle_multi_async_operation(host, community, hosts_report):
    try:
        start = time.perf_counter()
        os_info = await get_host_info_async(host, community)
        end = time.perf_counter()
        if os_info:
            click.echo(f'Getting packages for Host: {host}')
            start = time.perf_counter()
            packages = await get_packages_async(host, community, os_name=os_info.get('os_name').lower())
            end = time.perf_counter()
            hosts_report[host] = {
                "os": os_info,
                "ipv4": host,
                "packages": packages
            }
    except:
        pass



if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    
    hosts={}
    asyncio.run(handle_multi_async_operation('localhost','public',hosts))
    # print(hosts)


import logging
import os
import platform
import signal
import subprocess
import sys
import time
import re
from datetime import datetime, timedelta

import click
import psutil
from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.triggers.cron import CronTrigger

from watchman_agent_v2.core.utils.launch_detached import launch_detached


@click.group()
def cron():
    """Commandes pour gérer les tâches en arrière plan."""
    pass


# start
def get_executable_name():
    """Retourne le nom de l'exécutable utilisé (agent.exe ou watchman-agent)"""
    return os.path.basename(sys.argv[0])


@click.command()
@click.option('--hour', type=str, required=True, help="L'heure à laquelle démarrer le serveur (0-23).")
@click.option('--minute', type=str, required=True, help="La minute à laquelle démarrer le serveur (0-59).")
@click.option('--day', type=str, required=False, default="*", help="Jour du mois (1-31), '*' pour chaque jour.")
@click.option('--month', type=str, required=False, default="*", help="Mois (1-12), '*' pour chaque mois.")
@click.option('-d', '--detach', is_flag=True, help="Exécution de l'agent en arrière-plan.")
@click.option('--mode',default="local", type=click.Choice(['local', 'network']), help="Mode d'exécution de l'agent", required=False)
def start(hour, minute, day, month, detach,mode):
    """Configure la planification des tâches."""

    # Configuration des logs
    file_path = "watchman_agent_v2/logs/logs.log"
    directory = os.path.dirname(file_path)
    os.makedirs(directory, exist_ok=True)
    log_file = file_path
    logging.basicConfig(
        filename=log_file,
        level=logging.INFO,
        format="%(asctime)s - %(levelname)s - %(message)s",
        encoding="utf-8"
    )

    executable = get_executable_name()

    def run_agent_on_schedule():
        logging.info(f"🔄 TACHE_EXECUTEE >> {hour}:{minute} (Jour: {day}, Mois: {month})")
        try:
            with open(log_file, "a", encoding="utf-8") as f:
                subprocess.Popen(
                    [executable, "run", "--mode", mode] if mode else [executable, "run"],
                    stdout=f, stderr=subprocess.STDOUT,
                    close_fds=True,
                    start_new_session=True
                )
            logging.info(f"Agent démarré avec succès (logs: {log_file})")
        except Exception as e:
            logging.error(f"❌ Erreur lors du démarrage de l'agent: {e}")

    scheduler = BackgroundScheduler()
    trigger = CronTrigger(hour=hour, minute=minute, day=day, month=month)
    scheduler.add_job(run_agent_on_schedule, trigger=trigger, name="run_agent_on_schedule")

    if detach:
        logging.info("🛠 Démarrage du planificateur en arrière-plan...")

        cmd = [
            executable, "cron", "start",
            "--hour", hour,
            "--minute", minute,
            "--mode", mode
        ]
        if day != "*":
            cmd.extend(["--day", day])
        if month != "*":
            cmd.extend(["--month", month])

        logging.info(f"Commande exécutée : {' '.join(cmd)}")
        print(f"Commande exécutée : {' '.join(cmd)}")

        try:
            launch_detached(cmd, "cron", log_file)
        except Exception as e:
            logging.error(f"❌ Erreur lors de l'exécution de la commande cron: {e}")
        return

    logging.info("🟢 Planificateur en cours d'exécution...")
    scheduler.start()
    logging.info(f"Tâche planifiée pour {hour}:{minute} (Jour: {day}, Mois: {month})")

    try:
        while True:
            time.sleep(1)
    except (KeyboardInterrupt, SystemExit):
        scheduler.shutdown()
        logging.info("🛑 Planificateur arrêté.")

    click.echo(f"Tâche cron configurée")


# status
@click.command()
def status():
    """Affiche si le planificateur cron est en cours ou non."""
    pid_file = "watchman_agent_v2/pids/cron.pid"

    # click.echo("État du Planificateur :\n")

    if not os.path.exists(pid_file):
        click.echo("Arrêté.")
        return

    try:
        with open(pid_file, "r") as f:
            pid = int(f.read().strip())

        if psutil.pid_exists(pid):
            click.echo(f"En cours")
        else:
            click.echo("Arrêté")
    except Exception:
        click.echo("Impossible de lire le fichier PID.")


@click.command()
@click.option("--lines", default=30, help="Nombre de lignes à afficher depuis la fin du fichier de log.")
def logs(lines):
    """Affiche les derniers logs du planificateur."""
    log_file = "watchman_agent_v2/logs/logs.log"

    click.echo("Derniers logs :\n")

    if not os.path.exists(log_file):
        click.echo("Aucun fichier de log trouvé.")
        return

    try:
        with open(log_file, "r", encoding="utf-8", errors="ignore") as f:
            all_lines = f.readlines()
            for line in all_lines[-lines:]:
                click.echo(line.rstrip())
    except Exception as e:
        click.echo(f"Erreur lors de la lecture des logs : {e}")


# stop
@click.command()
def stop():
    """
    Arrête automatiquement toutes les tâches en arrière-plan (agent, planificateurs, etc.).
    Nettoie également les fichiers PID orphelins.
    """
    pid_dir = "watchman_agent_v2/pids/"

    if not os.path.isdir(pid_dir):
        click.echo("Aucun processus trouvé.")
        return

    pid_files = [f for f in os.listdir(pid_dir) if f.endswith(".pid")]
    running_tasks = {}

    for pid_file in pid_files:
        path = os.path.join(pid_dir, pid_file)
        try:
            with open(path, "r") as f:
                pid = int(f.read().strip())
            if psutil.pid_exists(pid):
                task_name = pid_file.replace(".pid", "")
                running_tasks[task_name] = (pid, path)
            else:
                os.remove(path)
                click.echo(f"Fichier PID orphelin supprimé : {pid_file}")
        except Exception as e:
            click.echo(f"Erreur lors du traitement de {pid_file} : {e}")

    if not running_tasks:
        click.echo("Aucun processus actif trouvé.")
        return

    for name, (pid, path) in running_tasks.items():
        try:
            os.kill(pid, signal.SIGTERM)
            os.remove(path)
            click.echo(f"Tâche '{name}' arrêtée (PID {pid})")
        except Exception as e:
            click.echo(f"Erreur lors de l'arrêt de '{name}' : {e}")


cron.add_command(start)
cron.add_command(status)
cron.add_command(logs)
cron.add_command(stop)

if __name__ == '__main__':
    cron()



from abc import ABC, abstractmethod
from watchman_agent_v2.core.config.config_manager import ConfigManager

class BaseAgent(ABC):
    """
    Classe abstraite pour les agents.
    """
    def __init__(self):
        self.config=ConfigManager()
        
        
    @abstractmethod
    def run(self):
        """Exécute la collecte d'informations."""
        pass

    @abstractmethod
    def configure(self):
        """Configure l'agent."""
        pass



import os
import click
import base64

from watchman_agent_v2.core.config.config_manager import ConfigManager


@click.command()
def generate_key():
    """
    Génère une clé de cryptographie AES-256 et l'enregistre dans le fichier .env.
    """
    config_manager=ConfigManager()
    config_manager.generate_key()
    


if __name__ == "__main__":
    generate_key()



import logging

class AuditLogger:
    """
    Logger spécialisé pour des logs chiffrés avec audit trail.
    """
    def __init__(self):
        self.logger = logging.getLogger("audit")
        self.setup_logger()

    def setup_logger(self):
        """Configure le logger avec les handlers appropriés."""
        handler = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)
        self.logger.setLevel(logging.INFO)

    def log(self, message: str):
        """Enregistre un message dans le log d'audit."""
        self.logger.info(message)




import re
from semver.version import Version as sem_version
from packaging import version as pkg_version


linux_version_pattern = re.compile(
    r"^"
    # epoch must start with a digit
    r"(\d+:)?"
    # upstream must start with a digit
    r"\d"
    r"("
    # upstream  can contain only alphanumerics and the characters . + -
    # ~ (full stop, plus, hyphen, tilde)
    r"[A-Za-z0-9\.\+\~\-]+"
    r"|"
    # If there is no debian_revision then hyphens are not allowed in version.
    r"[A-Za-z0-9\.\+\~]+-[A-Za-z0-9\+\.\~]+"
    r")?"
    r"$"
)
irregular_version_pattern = re.compile(r'\d+(\.\d+)*')

def parse_version(text):
    """ Semantic Versioning (SemVer)
     Date-based Versioning
     Alphanumeric or Custom Schemes
     Debian based version parser
     Ubuntu based version parser
     parse version with build:
    """
    if not text:
        return None

    if linux_version_pattern.match(text):
        match = linux_version_pattern.search(text)
        if match:
            version = match.group()
            if ":" in version:
                epoch, _, version = version.partition(":")
                epoch = int(epoch)
            else:
                epoch = 0

            if "-" in version:
                upstream, _, revision = version.rpartition("-")
            else:
                upstream = version
                revision = "0"

            version = upstream
            regex_matched = False

            if 'ubuntu' in version:
                match = irregular_version_pattern.search(version)
                if match:
                    regex_matched = True
                    version = match.group()
            elif 'debian' in version:
                match = irregular_version_pattern.search(version)
                if match:
                    regex_matched = True
                    version = match.group()
            elif 'git' in version:
                match = irregular_version_pattern.search(version)
                if match:
                    regex_matched = True
                    version = match.group()
            elif '-' in version:
                match = irregular_version_pattern.search(version)
                if match:
                    regex_matched = True
                    version = match.group()
            else:
                match = irregular_version_pattern.search(version)
                if match:
                    regex_matched = True
                    version = match.group()

            parsed = None
            if not regex_matched:
                try:
                    parsed = sem_version.parse(version)
                except ValueError:
                    try:
                        parsed = pkg_version.parse(version)
                    except pkg_version.InvalidVersion:
                        parsed = None

            if parsed:
                parsed_split_len = len(str(parsed).split("."))
                if parsed_split_len < 3:
                    version = [str(parsed.major), str(parsed.minor)]
                elif parsed_split_len == 3:
                    try:
                        version = [str(parsed.major), str(parsed.minor), str(parsed.patch)]
                    except AttributeError:
                        version = [str(parsed.major), str(parsed.minor), str(parsed.micro)]
                else:
                    version = parsed

                if isinstance(version, list):
                    version = ".".join(version)
                else:
                    version = version
            else:
                if not regex_matched:
                    print(f'Cannot definitely parse version {text}')
            return version


import logging
import ssl
from typing import Dict, List
import pythoncom
import watchman_agent_v2.core.protocols.wmi_protocol as wmi_protocol
from ldap3 import Server, Connection, Tls, ALL, SUBTREE

logger = logging.getLogger(__name__)

class SecureSystemInspector:
    """
    Classe combinant WMI (Windows Management Instrumentation) et LDAPS
    pour une collecte sécurisée d'informations système et AD.
    """
    
    def __init__(self, ldap_config: Dict, wmi_credentials: Dict = None):
        self.ldap_config = ldap_config
        self.wmi_credentials = wmi_credentials or {}
        self.ldap_conn = None
        self.wmi_conn = None

    def connect_ldaps(self):
        """Établit une connexion sécurisée LDAPS"""
        try:
            tls_config = Tls(
                validate=ssl.CERT_REQUIRED,
                version=ssl.PROTOCOL_TLSv1_2
            )
            server = Server(
                self.ldap_config['server'],
                use_ssl=True,
                tls=tls_config,
                get_info=ALL
            )
            self.ldap_conn = Connection(
                server,
                user=self.ldap_config['user'],
                password=self.ldap_config['password'],
                auto_bind=True
            )
            logger.info("Connexion LDAPS établie avec succès")
        except Exception as e:
            logger.error(f"Erreur LDAPS: {str(e)}")
            raise

    def connect_wmi(self):
        """Établit une connexion WMI sécurisée"""
        try:
            pythoncom.CoInitialize()
            self.wmi_conn = wmi_protocol.WMI(
                computer=self.wmi_credentials.get('host', 'localhost'),
                user=self.wmi_credentials.get('user'),
                password=self.wmi_credentials.get('password'),
                namespace='root\\Microsoft\\Windows\\Storage'
            )
            logger.info("Connexion WMI sécurisée établie")
        except Exception as e:
            logger.error(f"Erreur WMI: {str(e)}")
            raise

    def get_combined_system_info(self) -> Dict:
        """Combine les données WMI et AD"""
        return {
            'local_system': self.get_wmi_system_info(),
            'ad_data': self.get_ad_computer_info(),
            'software': self.get_installed_software(),
            'storage': self.get_secure_storage_info()
        }

    def get_wmi_system_info(self) -> Dict:
        """Récupère les informations système via WMI"""
        info = {}
        try:
            cs = self.wmi_conn.Win32_ComputerSystem()[0]
            os = self.wmi_conn.Win32_OperatingSystem()[0]
            info = {
                'manufacturer': cs.Manufacturer,
                'model': cs.Model,
                'domain': cs.Domain,
                'os_version': os.Caption,
                'last_boot': os.LastBootUpTime
            }
        except Exception as e:
            logger.error(f"Erreur WMI système: {str(e)}")
        return info

    def get_ad_computer_info(self) -> Dict:
        """Récupère les informations AD via LDAPS"""
        search_base = f"OU=Computers,{self.ldap_config['base_dn']}"
        search_filter = f"(cn={self.wmi_credentials.get('host', '*')})"
        
        try:
            self.ldap_conn.search(
                search_base=search_base,
                search_filter=search_filter,
                search_scope=SUBTREE,
                attributes=['*']
            )
            return self.ldap_conn.entries[0].entry_attributes_as_dict
        except Exception as e:
            logger.error(f"Erreur AD: {str(e)}")
            return {}

    def get_installed_software(self) -> List[Dict]:
        """Liste les logiciels installés de manière sécurisée"""
        software = []
        try:
            for pkg in self.wmi_conn.Win32_Product():
                software.append({
                    'name': pkg.Name,
                    'version': pkg.Version,
                    'vendor': pkg.Vendor,
                    'install_date': pkg.InstallDate
                })
        except Exception as e:
            logger.error(f"Erreur logiciels: {str(e)}")
        return software

    def get_secure_storage_info(self) -> Dict:
        """Récupère les informations de stockage chiffrées"""
        try:
            disks = {}
            for disk in self.wmi_conn.MSFT_PhysicalDisk():
                disks[disk.DeviceId] = {
                    'model': disk.Model,
                    'size_GB': int(disk.Size) // (1024**3) if disk.Size else 0,
                    'encrypted': disk.IsEncrypted,
                    'health_status': disk.HealthStatus
                }
            return disks
        except Exception as e:
            logger.error(f"Erreur stockage: {str(e)}")
            return {}

    def __del__(self):
        """Nettoyage sécurisé des connexions"""
        if self.ldap_conn:
            self.ldap_conn.unbind()
        if self.wmi_conn:
            pythoncom.CoUninitialize()

# Exemple d'utilisation
if __name__ == "__main__":
    # Configuration
    LDAP_CONFIG = {
        'server': '10.10.15.100',
        'base_dn': 'dc=GITS,dc=BJ',
        'user': 'GITS\\Bienvenu',
        'password': 'Welcom@94513830&'
    }

    WMI_CREDS = {
        'host': '10.10.15.100',
        'user': 'GITS\\Bienvenu',
        'password': 'Welcom@94513830&'
    }

    # Initialisation
    inspector = SecureSystemInspector(LDAP_CONFIG, WMI_CREDS)
    
    try:
        inspector.connect_ldaps()
        inspector.connect_wmi()
        
        system_data = inspector.get_combined_system_info()
        
        # Affichage des résultats
        print("=== Informations Système ===")
        print(f"Modèle: {system_data['local_system']['model']}")
        print(f"Domaine AD: {system_data['ad_data']['distinguishedName']}")
        
        print("\n=== Logiciels Installés ===")
        for soft in system_data['software'][:5]:
            print(f"- {soft['name']} ({soft['version']})")

    except Exception as e:
        print(f"Erreur critique: {str(e)}")


from watchman_agent_v2.core.config.config_manager import ConfigManager
from watchman_agent_v2.core.services.api.assets_api import AssetAPIClient
from watchman_agent_v2.core.services.api.auth_api import AuthClient
from watchman_agent_v2.core.services.api.exceptions import AuthenticationError
from watchman_agent_v2.core.utils.host.manage_ad import ManageAd
from watchman_agent_v2.core.utils.log_manager import LogManager
from watchman_agent_v2.core.utils.host.manage_network_host import ManageNetworkHost

class AdAssets:
    
    def __init__(self):
        self.config=ConfigManager()
        
        
    def run(self):
        """Exécute la collecte d'informations en mode réseau."""
        # Implémenter la logique de collecte réseau
        self.setup_agent()

    def configure(self):
        """Configure le NetworkAgent."""
        pass
    
    def setup_agent(self):
        
        try:
            LogManager.info('Connexion à AD ....')
            try:
                client_id=self.config.config['api']['client_id']
                print(f'client_id {client_id} ')
                client_secret=self.config.decrypt(self.config.config['api']['client_secret'])
            except Exception as e:
                LogManager.error(f"Erreur lors de la récupération des informations d'authentification: {e}")
                return
            auth_client=AuthClient(
            client_id=client_id,
            client_secret=client_secret
             )
            
            token=''
            
            if auth_client.test_connection():
                token = auth_client.get_token()
                LogManager.success(f'Authentification réussie')
            else:
                LogManager.error(f"Impossible de se connecter au service d'authentification ")
                raise AuthenticationError()
            
            LogManager.info('Recuperations des adresses ....')
            
            LogManager.info(f'Recuperations des utilisateurs ....')
            # recuperations des configurations
            ad_config= self.config.config['ad']
            if all(ad_config.get(attr) for attr in ['ldap_server', 'ldap_port', 'search_base','user','password']):
                LogManager.info(f'Recuperations des valeurs des configs ....')
            else:
                LogManager.error(f'Configuration LDAP invalide ....')
                return
            LDAP_SERVER = ad_config['ldap_server']  # ou "ldaps://domaine.local" si vous utilisez LDAPS
            LDAP_PORT = int(ad_config['ldap_port'])                      # 389 pour LDAP ou 636 pour LDAPS
            USER_DN =  self.config.decrypt(ad_config['user'])            # Format (ex: "DOMAINE\\utilisateur")
            PASSWORD = self.config.decrypt(ad_config['password'])
            SEARCH_BASE = ad_config['search_base']
            # GROUP_NAME peut être None pour récupérer tous les utilisateurs
            GROUP_NAME = ad_config.get('group',None)
            DOMAIN = ad_config.get('domain',"")
            
            LDAP_USER = USER_DN
            LDAP_PASSWORD = PASSWORD
            # Optionnel : DN du groupe pour filtrer les machines
            GROUP_DN = GROUP_NAME  # ex: "CN=NomDuGroupe,OU=Groupes,DC=domaine,DC=local"

            # Paramètres pour la connexion distante (WMI ou WinRM)
            REMOTE_USER = LDAP_USER
            REMOTE_PASSWORD = LDAP_PASSWORD

            manager = ManageAd(ldap_server=LDAP_SERVER,
                            ldap_user=LDAP_USER,
                            ldap_password=LDAP_PASSWORD,
                            search_base=SEARCH_BASE,
                            domain=DOMAIN,
                            remote_user=REMOTE_USER,
                            remote_password=REMOTE_PASSWORD)

            if manager.connect_ldap():
                manager.fetch_computers(group_dn=GROUP_DN)
                manager.fetch_installed_applications()
                # Export des résultats
                path='watchman_agent_v2/data/json/watchman_system_info.json'
                manager.export_to_json(path)
                asset_api=AssetAPIClient(credentials={
                    "AGENT-ID": client_id,
                    "AGENT-SECRET": client_secret})
            # manage_host.export_to_csv('watchman_agent_v2/data/csv/watchman_system_info.csv')
            # to json
                LogManager.info(f'Envoie des informations vers Watchman ....')
            
                file_path='watchman_agent_v2/data/json/watchman_system_info.json'
                is_send, report, error_message = asset_api.send_assets(file_path)

                if is_send:
                    success_msg = (
                        f"Envoi réussi de {report.get('total', 0)} actifs | "
                        f"Succès: {report.get('success', 0)}/{report.get('total', 0)} | "
                        f"Taux: {report.get('success', 0)/max(report.get('total', 1), 1)*100:.1f}%"
                    )
                    LogManager.success(success_msg)
                    
                    # Log détaillé des erreurs si besoin
                    if report.get('failures', 0) > 0:
                        error_details = "\n".join(
                            [f"[{err['asset']}] {err['error']}" 
                            for err in report.get('errors', [])]
                        )
                        LogManager.error(f"Erreurs mineures détectées (non bloquantes):\n{error_details}")

                else:
                    error_msg = (
                        f"Échec de l'envoi - {error_message} | "
                        f"Rapport partiel: {report.get('success', 0)}/{report.get('total', 0)} réussis" 
                        if report 
                        else f"Échec complet - Aucun rapport disponible"
                    )
                    LogManager.error(error_msg)
                    
                    # Log supplémentaire si on a des détails d'erreur
                    if report and report.get('errors'):
                        errors_str = "\n".join(
                            [f"- {err['asset']}: {err['error'][:100]}..." 
                            for err in report.get('errors', [])]
                        )
                        LogManager.error(f"Détails des erreurs:\n{errors_str}")
            else :
                LogManager.error(f'Nous n\'avons pas pu trouver')
                
        except Exception as error:
            LogManager.error(f'Erreur {error}')
        
        
        



import click
import warnings

from watchman_agent_v2.core.utils.log_manager import LogManager
warnings.filterwarnings("ignore")
import sys
import io

sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding='utf-8')
from watchman_agent_v2.core.commands import configure, run, generate_key, active_directory, cron


@click.group()
def cli():
    """Agent CLI for network and local system scanning."""
    pass

try:
    cli.add_command(run.run, name="run")
    cli.add_command(cron.cron, name="cron")
    cli.add_command(configure.configure, name="configure")
    cli.add_command(generate_key.generate_key, name="generate-key")
    cli.add_command(active_directory.ad, name="ad")
except Exception as error:
    LogManager.error(f"Une erreur s'est produite {error} ")

if __name__ == "__main__":
    cli()



import re
import logging
import asyncio
from typing import Dict, Any, List
from datetime import datetime

import getmac
from puresnmp import Client, V2C, PyWrapper

from watchman_agent_v2.core.utils.extract_snmp_info import ExtractSnmpInfo
from watchman_agent_v2.core.utils.functions import parse_version


logger = logging.getLogger(__name__)

class SNMPHandler():
    """
    Handler SNMP pour la collecte d'informations système et logicielles.
    Utilise puresnmp pour interroger un hôte via SNMP.
    """
    
    # Définition des OIDs standard et étendu
    OIDS = {
        "sys_descr": "1.3.6.1.2.1.1.1.0",                   # Description système
        "sys_name": "1.3.6.1.2.1.1.5.0",                    # Nom d'hôte
        "if_phys_address": "1.3.6.1.2.1.2.2.1.6",           # Adresse MAC
        "hr_sw_installed": "1.3.6.1.2.1.25.6.3.1.2",        # Logiciels installés (standard)
        "hr_sw_installed_ext": "1.3.6.1.4.1.8072.1.3.2"     # Logiciels installés étendu (Linux)
    }
    
    def __init__(self, config: Dict[str, Any] = None):
        """
        Initialise le gestionnaire SNMP avec la configuration donnée.
        """
        super().__init__()
        self.config = config or {}
        self.community = self.config.get("community", "public")
        self.port = self.config.get("port", 161)
        self.timeout = self.config.get("timeout", 3)
        self.version = self.config.get("version", "2c")
    
    def collect_info(self, ip: str, config: Dict[str, Any] = None) -> Dict[str, Any]:
         return asyncio.run(self.get_info(ip,config))
    
    async def get_info(self, ip: str, config: Dict[str, Any] = None) -> Dict[str, Any]:
        """
        Collecte et retourne les informations SNMP complètes depuis l'hôte cible.
        Ajoute également le timestamp d'accès.
        """
        if config:
            self.config = config
            self.community = config.get("community", "public")
            self.port = config.get("port", 161)
            self.timeout = config.get("timeout", 3)
            self.version = config.get("version", "2c")
            self.ip=ip
        
        # Initialisation du client SNMP pour l'IP cible
        self.client = PyWrapper(
            Client(ip=ip, credentials=V2C(self.community), port=self.port)
        )

        result: Dict[str, Any] = {
            'system_info': {
                "ip": ip,
                "mac": "",
                "architecture": "",
                "os": "",
                "hostname": "",
            },
            "applications": [],
            "accessed_at": ""  # Timestamp d'accès
        }
        
        try:
            # Récupération des informations de base
            basic_info = await self._get_basic_info()
            self.basic_info=basic_info
            result['system_info'].update(basic_info)
            
            # Récupération des applications installées
            result["applications"] = await self._get_installed_software()
            
            # Récupération de l'adresse MAC et mise à jour dans system_info
            mac = await self._get_mac_address()
            result['system_info']["mac"] = mac
            
            # Ajout du timestamp d'accès
            result["accessed_at"] = datetime.now().isoformat()
            
        except asyncio.TimeoutError:
            logger.warning(f"Timeout SNMP sur {ip}")
        except Exception as e:
            logger.error(f"Erreur SNMP sur {ip}: {str(e)}")
            
        return result
    
    async def _get_basic_info(self) -> Dict[str, str]:
        """
        Récupère les informations système de base.
        Retourne un dictionnaire contenant l'OS, l'architecture et le hostname.
        """
        info: Dict[str, str] = {}
        try:
            # Récupération de la description système
            sys_descr = await self.client.get(self.OIDS["sys_descr"])
            # Décodage si nécessaire
            if isinstance(sys_descr, bytes):
                sys_descr = sys_descr.decode('utf-8', errors='replace')
            print(f'sys_info {sys_descr} ')
            lower_value=sys_descr.lower()
            if "windows" in lower_value:
                info = ExtractSnmpInfo.extract_windows_host_info(sys_descr)
                info["os"] = info.get("os_name")
                info["kernel_version"] = info.get("kernel_version")
                info["architecture"] = info.get("arch")
                info["build"] = info.get("build")
            elif "darwin" in lower_value:
                partitioned = sys_descr.split()
                if partitioned:
                    info["os"] = partitioned[0]
                    info["kernel_version"] = partitioned[2]
                    info["architecture"] = partitioned[-1]
            elif "linux" in lower_value:
                partitioned = sys_descr.split()
                if partitioned:
                    info["os"] = partitioned[0]
                    info["kernel_version"] = partitioned[2]
                    info["architecture"] = partitioned[-1]
            else :
                info["os"] = self._parse_os(sys_descr)
                info["architecture"] = self._parse_architecture(sys_descr)
        
            # Récupération du nom d'hôte
            hostname = await self.client.get(self.OIDS["sys_name"])
            if isinstance(hostname, bytes):
                hostname = hostname.decode('utf-8', errors='replace')
            info["hostname"] = hostname
            info["mac"] = getmac.get_mac_address(ip=self.ip)
            
        except Exception as e:
            logger.error(f"Erreur lors de la collecte des informations de base: {str(e)}")
        return info
    
    async def _get_installed_software(self) -> List[Dict[str, str]]:
        """
        Récupère la liste des logiciels installés.
        Tente d'abord avec l'OID étendu pour obtenir nom et version directement,
        sinon retombe sur l'OID standard et tente de parser la version depuis le nom.
        Chaque entrée est un dictionnaire avec le nom, la version et le fournisseur.
        """
        applications: List[Dict[str, str]] = []
        # Première tentative avec l'OID étendu
        try:
            ext_entries = []
            async for entry in self.client.walk(self.OIDS["hr_sw_installed_ext"]):
                ext_entries.append(entry)
            if ext_entries:
                # Exemple de chaîne : b'xkb-data\t2.33-1'
                for entry in ext_entries:
                    if isinstance(entry.value, bytes):
                        decoded = entry.value.decode('utf-8', errors='replace')
                        parts = decoded.split('\t')
                        if len(parts) >= 2:
                            app_info = {
                                "name": parts[0].strip(),
                                "version": parts[1].strip(),
                                "vendor": "*",  # Vendor non fourni par cet OID
                                "type": "application"
                            }
                            applications.append(app_info)
                if applications:
                    return applications
        except Exception as e:
            logger.error(f"Erreur lors de la collecte avec OID étendu: {str(e)}")
        
        # Fallback sur l'OID standard
        try:
            async for entry in self.client.walk(self.OIDS["hr_sw_installed"]):
                # Selon la version de puresnmp, la structure peut varier.
                raw_value = getattr(entry, "raw_value", {}) or getattr(entry, "value", {})
                if type(raw_value) == int:
                    continue
                name = raw_value
                if isinstance(name, bytes):
                    name = name.decode('utf-8', errors='replace')
                
                extract_fnc =  ExtractSnmpInfo.extract_info_windows
                if self.basic_info['os'].lower() == "linux":
                    extract_fnc = ExtractSnmpInfo.extract_info_linux
                elif self.basic_info['os'].lower() == "mac" or self.basic_info['os'].lower() == "darwin":
                    extract_fnc = ExtractSnmpInfo.extract_info_macos
                    
                host_info=extract_fnc(name)
                version=host_info['version']
                name=host_info['name']
                vendor="*"
                
                app_info = {
                    "name": name,
                    "version": parse_version(version),
                    "vendor": vendor,
                    "type": "application"
                }
                if app_info["name"]:
                    applications.append(app_info)
        except Exception as e:
            logger.error(f"Erreur lors de la collecte des logiciels installés avec l'OID standard: {str(e)}")
        return applications
    
    async def _get_mac_address(self) -> str:
        """
        Parcourt les interfaces et retourne la première adresse MAC non-nulle.
        """
        mac_address: str = ""
        try:
            async for entry in self.client.walk(self.OIDS["if_phys_address"]):
                mac_bytes = entry.value
                if mac_bytes and isinstance(mac_bytes, bytes):
                    mac = ":".join(f"{b:02x}" for b in mac_bytes)
                    if mac != "00:00:00:00:00:00":
                        return mac
                elif mac_bytes and isinstance(mac_bytes, str) and mac_bytes.strip():
                    return mac_bytes.strip()
        except Exception as e:
            logger.error(f"Erreur lors de la collecte de l'adresse MAC: {str(e)}")
        return mac_address
    
    @staticmethod
    def _parse_os(sys_descr: str) -> str:
        """
        Extrait le nom de l'OS depuis la description système.
        """
        patterns = [
            r"Linux\s+([^\s]+)",
            r"Windows\s+([^\s]+)",
            r"Darwin/([^\s]+)",  # macOS
            r"([A-Za-z]+)\s+UNIX"
        ]
        
        for pattern in patterns:
            match = re.search(pattern, sys_descr)
            if match:
                return match.group(1)
        return sys_descr.split()[0] if sys_descr else ""
    
    @staticmethod
    def _parse_architecture(sys_descr: str) -> str:
        """
        Extrait l'architecture depuis la description système.
        """
        patterns = {
            r"x86_64": "x86_64",
            r"amd64": "x86_64",
            r"i[3456]86": "x86",
            r"armv[^\s]+": "ARM",
            r"aarch64": "ARM64"
        }
        
        for pattern, arch in patterns.items():
            if re.search(pattern, sys_descr, re.IGNORECASE):
                return arch
        return "unknown"



# Exemple d'utilisation asynchrone
if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    
    config = {
        "community": "public",
        "version": "2c",
        "timeout": 5
    }
    
    var_binds = {
        'sys_hostname_bind': '1.3.6.1.2.1.1.5.0',
        'sys_descr_bind': '1.3.6.1.2.1.1.1.0',
    }
    
    def main():
        handler = SNMPHandler()
        info =  handler.collect_info("10.10.15.100",config)
        return info
    
    print(main())
    
    # async def example():
    #     client = PyWrapper(Client("localhost", V2C("public")))
    #     output = []
        
        
    #     async for item in client.walk("1.3.6.1.2.1.25.6.3.1.2"):
    #         output.append(item.value)
    #     return output

    # print(asyncio.run(example()))



from impacket.dcerpc.v5.dcomrt import DCOMConnection
from impacket.dcerpc.v5.dcom.wmi import CLSID_WbemLevel1Login, IWbemLevel1Login
from impacket.dcerpc.v5.dtypes import NULL
import socket
import logging

class WMICredentialValidator:
    def __init__(self, timeout=10):
        self.timeout = timeout
        self.logger = logging.getLogger('wmi_validator')

    def validate(self, domain, username, password, client_ip):
        """Valide des credentials WMI contre une machine cible"""
        try:
            return self._attempt_wmi_connection(
                domain=domain,
                username=username,
                password=password,
                target=client_ip
            )
        except socket.error as e:
            self.logger.error(f"Erreur réseau : {str(e)}")
            return False
        except Exception as e:
            self.logger.debug(f"Erreur d'authentification : {str(e)}")
            return False

    def _attempt_wmi_connection(self, domain, username, password, target):
        """Tente une connexion WMI via DCOM avec timeout"""
        dcom = None
        try:
            dcom = DCOMConnection(
                target,
                username=username,
                password=password,
                domain=domain,
                oxidResolver=True,
                timeout=self.timeout,
                doKerberos=False
            )

            # Tente d'accéder au namespace root/cimv2
            iInterface = dcom.CoCreateInstanceEx(CLSID_WbemLevel1Login)
            iWbemLevel1Login = IWbemLevel1Login(iInterface)
            iWbemLevel1Login.NTLMLogin('root\\cimv2', NULL, NULL)
            
            # Nettoyage propre
            iWbemLevel1Login.RemRelease()
            dcom.disconnect()
            return True

        except Exception as e:
            if dcom:
                dcom.disconnect()
            raise e


from watchman_agent_v2.core.utils.log_manager import LogManager


class ProtocolSelector:
    def __init__(self, protocol_handlers: dict, protocols_config: dict):
        self.protocol_handlers = protocol_handlers
        self.protocols = protocols_config
        self.logger = LogManager

    def deep_merge(self, base: dict, update: dict) -> dict:
        """
        Fusion récursive de dictionnaires.
        Combine les configurations globales et les surcharges intelligemment.
        """
        merged = base.copy()
        for key, val in update.items():
            if isinstance(val, dict) and key in merged:
                merged[key] = self.deep_merge(merged.get(key, {}), val)
            else:
                merged[key] = val
        return merged

    def get_protocols(self, address: str) -> dict:
        """
        Retourne la configuration complète des protocoles pour une adresse,
        en conservant TOUS les protocoles globaux et leurs paramètres,
        même si non mentionnés dans les surcharges.
        """
        global_config = self.protocols.get('global', {})
        address_overrides = self.protocols.get('overrides', {}).get(address, {})
        return self.deep_merge(global_config, address_overrides)

    def select_and_run(self, address: str) -> tuple:
        """
        Sélectionne et exécute le premier protocole activé.
        Retourne (résultat, nom du protocole, instance handler)
        """
        protocols_config = self.get_protocols(address)
        self.logger.info(f"Protocoles configurés pour {address}")

        for protocol_name, protocol_config in protocols_config.items():
            if not protocol_config.get('enabled', False):
                continue

            try:
                handler = self._get_handler(protocol_name)
                result = handler.collect_info(address, protocol_config)
                return result, protocol_name, handler
            except Exception as e:
                self.logger.error(f"Erreur avec {protocol_name} sur {address}: {str(e)}")
                continue

        raise Exception(f"Aucun protocole n'a réussi pour {address}")

    def _get_handler(self, protocol_name: str):
        handler_class = self.protocol_handlers.get(protocol_name)
        if not handler_class:
            raise ValueError(f"Handler non trouvé pour le protocole: {protocol_name}")
        return handler_class()



from abc import ABC, abstractmethod

from watchman_agent_v2.core.config.config_manager import ConfigManager


class BaseProtocol(ABC):
    """
    Implémentation spécifique du protocole SNMP.
    """

    def __init__(self):
        self.config = ConfigManager()

    



import platform
import logging
import csv
import json
import sys
from typing import List, Dict, Any
from ldap3 import Server, Connection, ALL, SUBTREE, NTLM
import socket
from watchman_agent_v2.core.protocols.http import HttpProtocol
from watchman_agent_v2.core.protocols.snmp import SNMPHandler
from watchman_agent_v2.core.protocols.ssh import SSHProtocol
from watchman_agent_v2.core.protocols.wmi_protocol import WMIProtocol
from watchman_agent_v2.core.utils.host.manage_host import ManageHost
from watchman_agent_v2.core.utils.log_manager import LogManager
from watchman_agent_v2.core.utils.protocol_selector import ProtocolSelector

# Configuration du logging
logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s - %(levelname)s - %(message)s')


class ManageAd(ManageHost):
    def __init__(self,
                 ldap_server: str,
                 ldap_user: str,
                 ldap_password: str,
                 search_base: str,
                 domain: str,
                 remote_user: str,
                 remote_password: str):
        """
        Initialise la connexion LDAP et stocke les informations d'authentification pour l'accès distant.
        :param ldap_server: URL ou IP du contrôleur de domaine (ex: 'ldap://dc.domaine.local')
        :param ldap_user: Utilisateur LDAP (ex: 'DOMAIN\\username')
        :param ldap_password: Mot de passe LDAP
        :param search_base: Base de recherche LDAP (ex: 'DC=domaine,DC=local')
        :param domain: Nom de domaine (pour WMI ou WinRM)
        :param remote_user: Identifiant pour se connecter aux machines distantes (généralement identique à ldap_user)
        :param remote_password: Mot de passe associé
        """
        super().__init__()
        self.ldap_server = ldap_server
        self.ldap_user = ldap_user
        self.ldap_password = ldap_password
        self.search_base = search_base
        self.domain = domain
        self.remote_user = remote_user
        self.remote_password = remote_password

        self.conn = None  # Connexion LDAP
        self.logger = LogManager
        self.results: Dict[str, Any] = {'devices': {}}
        self.addresses: List[str] = []
        # networks data 
        self.exclusions = self.config.config['network'].get('exclusions',[])
        self.protocols = self.config.config['network'].get('protocols')
        self.protocol_handlers={
                    'http': HttpProtocol,
                    'snmp': SNMPHandler,
                    'ssh':SSHProtocol,
                    'wmi': WMIProtocol
                        }
        
        self.protocol_selector = ProtocolSelector(self.protocol_handlers,self.protocols)
   
    def connect_ldap(self) -> bool:
        """
        Établit la connexion au serveur LDAP.
        """
        try:
            server = Server(self.ldap_server, get_info=ALL)
            self.conn = Connection(server,
                                   user=self.ldap_user,
                                   password=self.ldap_password,
                                   authentication='SIMPLE',
                                   auto_bind=True)
            self.logger.info("Connexion LDAP établie avec succès")
            return True
        except Exception as e:
            self.logger.error(f"Erreur de connexion LDAP : {e}")
            return False

    def fetch_computers(self, group_dn: str = None) -> None:
        """
        Récupère les machines du domaine ou d'un groupe spécifique.
        :param group_dn: DN du groupe LDAP (optionnel). Si None, tous les objets de type computer sont recherchés.
        """
        if not self.conn:
            self.logger.error("La connexion LDAP n'est pas établie")
            return

        try:
            if group_dn:
                # Recherche des ordinateurs membres d'un groupe spécifique
                search_filter = f'(&(objectCategory=computer)(memberOf={group_dn}))'
            else:
                # Recherche de tous les objets de type computer
                search_filter = '(objectCategory=computer)'

            self.conn.search(search_base=self.search_base,
                             search_filter=search_filter,
                             search_scope=SUBTREE,
                             attributes=['cn', 'dNSHostName'])
            self.logger.info(f"{len(self.conn.entries)} ordinateur(s) trouvé(s)")
            for entry in self.conn.entries:
                # On privilégie le dNSHostName sinon le cn
                hostname = str(entry.dNSHostName) if entry.dNSHostName.value else str(entry.cn)
                
                if hostname:
                    try:
                        ip = socket.gethostbyname(hostname)
                    except socket.gaierror:
                        self.logger.warning(f"Impossible de résoudre {hostname}")
                        ip=hostname
                    self.addresses.append(ip)
                    print(hostname)
                    self.results['devices'][ip] = {
                        'system_info': {'hostname': hostname,"ip":ip},
                        'applications': []
                    }
            if not self.addresses:
                self.logger.warning("Aucun ordinateur trouvé.")
            self.addresses = [
                ip for ip in self.addresses 
                if ip not in self.exclusions
            ]
        except Exception as e:
            self.logger.error(f"Erreur lors de la récupération des machines : {e}")

    def fetch_installed_applications(self) -> None:
        """
        Pour chaque machine récupérée, se connecte en remote et extrait la liste des applications installées.
        Utilise WMI sous Windows ou WinRM sous Linux.
        """
        current_os = platform.system().lower()
        self.logger.info(f"Système d'exécution détecté : {current_os}")
        for hostname in self.addresses:
            self.logger.info(f"Récupération des applications sur : {hostname}")
            apps = []
            system_info = {}
            # device_results, selected_protocol, handler = self.protocol_selector.select_and_run(hostname)
            if current_os == 'windows':
                # Utilisation de WMI pour Windows
                try:
                    import wmi
                    c = wmi.WMI(computer=hostname,
                                user=self.remote_user,
                                password=self.remote_password)
                    for product in c.Win32_Product():
                        app = {
                            'name': product.Name,
                            'version': product.Version,
                            'vendor': product.Vendor,
                            'type': 'application'  # À adapter si besoin
                        }
                        apps.append(app)
                    for os in c.Win32_OperatingSystem():
                        system_info['os'] = os.Caption
                        system_info['os_version'] = os.Version
                        system_info['architecture'] = os.OSArchitecture

                    for nic in c.Win32_NetworkAdapterConfiguration(IPEnabled=True):
                        if nic.MACAddress:
                            system_info['mac'] = nic.MACAddress
                            break
                    # add os as apps 
                    apps.append({
                            'name':system_info['os'],
                            'version':system_info['os_version'],
                            "vendor":"*",
                            "type":"os"})
                    self.logger.info(f"{len(apps)} application(s) trouvée(s) sur {hostname} via WMI")
                except Exception as e:
                    self.logger.error(f"Erreur lors de la connexion WMI sur {hostname}: {e}")
            else:
                # Utilisation de WinRM pour Linux (ou autres OS non Windows)
                try:
                    import winrm
                    # Construit l'URL WinRM pour la machine distante
                    # Le port 5985 est utilisé par défaut pour WinRM en HTTP
                   
                    # Construit l'URL WinRM pour la machine distante
                    winrm_url = f'http://{hostname}:5985/wsman'
                    session = winrm.Session(winrm_url, auth=(self.remote_user, self.remote_password), transport='ntlm')
                    self.logger.info('connexion success')
                    # La commande PowerShell pour récupérer les applications installées et infos système
                    ps_script = r"""
$ProgressPreference = 'SilentlyContinue'
$ErrorActionPreference = 'Continue'

# Infos système
$sysinfo = @{    os           = (Get-CimInstance Win32_OperatingSystem).Caption
    os_version   = (Get-CimInstance Win32_OperatingSystem).Version
    architecture = (Get-CimInstance Win32_OperatingSystem).OSArchitecture
    mac          = (Get-CimInstance Win32_NetworkAdapterConfiguration |
                    Where-Object { $_.MACAddress -and $_.IPEnabled } |
                    Select-Object -First 1).MACAddress
}

# Liste des applications installées
$appList = Get-CimInstance Win32_Product |
           Select-Object Name, Version, Vendor

# Conversion en JSON, suppression du flux d'erreur et sortie complète
@{
    system_info  = $sysinfo
    applications = $appList
} |
ConvertTo-Json -Depth 3 2>$null |
Out-String -Width 4096
"""
                    result = session.run_ps(ps_script)
                    if result.status_code == 0:
                        # La sortie est en JSON
                        data = json.loads(result.std_out.decode('utf-8'))
                        
                        # Extraire les infos
                        system_info = data.get('system_info', {})
                        apps_data = data.get('applications', [])

                        # Si un seul objet est retourné, le mettre dans une liste
                        if isinstance(apps_data, dict):
                            apps_data = [apps_data]

                        apps = [{
                            'name': app.get('Name', ''),
                            'version': app.get('Version', ''),
                            'vendor': app.get('Vendor', ''),
                            'type': 'application'
                        } for app in apps_data if app.get('Name')]
                        # add os as apps 
                        apps.append({
                            'name':system_info['os'],
                            'version':system_info['os_version'],
                            "vendor":"*",
                            "type":"os"})
                        self.logger.info(f"{len(apps)} application(s) trouvée(s) sur {hostname} via WinRM")
                    else:
                        err = result.std_err.decode('utf-8')
                        self.logger.error(f"Erreur WinRM sur {hostname}: {err}")
                except Exception as e:
                    self.logger.error(f"Erreur lors de la connexion WinRM sur {hostname}: {e}")

            # Ajouter les informations récupérées (applications + system_info)
            self.results['devices'][hostname]['applications'] = apps
            self.results['devices'][hostname]['system_info'].update(system_info)



    def export_to_csv(self, filename: str = 'applications.csv') -> None:
        """
        Exporte les informations collectées au format CSV.
        """
        if not self.results['devices']:
            self.logger.error("Veuillez récupérer d'abord les informations")
            return
        try:
            with open(filename, 'w', newline='', encoding='utf-8') as csvfile:
                writer = csv.writer(csvfile, quoting=csv.QUOTE_MINIMAL)
                # En-têtes du CSV
                headers = [
                    'hostname', 'ip', 'mac', 'os', 'architecture',
                    'application_name', 'application_version', 'vendor', 'type'
                ]
                writer.writerow(headers)
                for hostname, data in self.results['devices'].items():
                    sys_info = data.get('system_info', {})
                    applications = data.get('applications', [])
                    ip = sys_info.get('ip', '')
                    mac = sys_info.get('mac', '')
                    os_name = sys_info.get('os', '')
                    arch = sys_info.get('architecture', '')
                    for app in applications:
                        row = [
                            hostname,
                            ip,
                            mac,
                            os_name,
                            arch,
                            app.get('name', ''),
                            app.get('version', ''),
                            app.get('vendor', ''),
                            app.get('type', '')
                        ]
                        writer.writerow(row)
            self.logger.info(f"CSV généré avec succès : {filename}")
        except Exception as e:
            self.logger.error(f"Erreur lors de l'export CSV: {e}")

    def export_to_json(self, filename: str = 'system_info.json') -> None:
        """
        Exporte les informations collectées au format JSON.
        """
        if not self.results['devices']:
            self.logger.error("Veuillez récupérer d'abord les informations")
            return
        try:
            data_array = {'assets': []}
            for hostname, data in self.results['devices'].items():
                sys_info = data.get('system_info', {})
                asset = {
                    'hostname': sys_info.get('hostname', ''),
                    'ip': sys_info.get('ip', ''),
                    'mac': sys_info.get('mac', ''),
                    'os': sys_info.get('os', ''),
                    'architecture': sys_info.get('architecture', ''),
                    'applications': data.get('applications', [])
                }
                data_array['assets'].append(asset)
            with open(filename, 'w', encoding='utf-8') as jsonfile:
                json.dump(data_array, jsonfile, indent=4, ensure_ascii=False)
            self.logger.info(f"JSON généré avec succès : {filename}")
        except Exception as e:
            self.logger.error(f"Erreur lors de l'export JSON: {e}")


# Exemple d'utilisation
if __name__ == "__main__":
    # Paramètres LDAP et domaine
    LDAP_SERVER = "ldap://10.10.15.100"  # ou "ldaps://domaine.local" si vous utilisez LDAPS
    LDAP_PORT = 389                       # 389 pour LDAP ou 636 pour LDAPS
    USER_DN = "GITS\\Bienvenu"            # Format (ex: "DOMAINE\\utilisateur")
    PASSWORD = "Welcom@94513830&"
    # GROUP_NAME peut être None pour récupérer tous les utilisateurs
    LDAP_USER = USER_DN
    LDAP_PASSWORD = PASSWORD
    SEARCH_BASE = "DC=GITS,DC=BJ"
    # Optionnel : DN du groupe pour filtrer les machines
    GROUP_DN = None  # ex: "CN=NomDuGroupe,OU=Groupes,DC=domaine,DC=local"

    # Paramètres pour la connexion distante (WMI ou WinRM)
    REMOTE_USER = LDAP_USER
    REMOTE_PASSWORD = LDAP_PASSWORD

    manager = ManageAd(ldap_server=LDAP_SERVER,
                       ldap_user=LDAP_USER,
                       ldap_password=LDAP_PASSWORD,
                       search_base=SEARCH_BASE,
                       domain="GITS.BJ",
                       remote_user=REMOTE_USER,
                       remote_password=REMOTE_PASSWORD)

    if manager.connect_ldap():
        manager.fetch_computers(group_dn=GROUP_DN)
        manager.fetch_installed_applications()
        
        # Export des résultats
        manager.export_to_json('system_info.json')



from abc import ABC
import platform
import csv
import json
import subprocess
import socket
import uuid
import sys
from typing import List, Dict, Optional, Tuple
import psutil

from watchman_agent_v2.core.config.config_manager import ConfigManager

class ManageHost(ABC):
    def __init__(self):
        self.config=ConfigManager()
        



class Scheduler:
    """
    Scheduler pour gérer les cron jobs via Celery ou APScheduler.
    """
    def __init__(self, schedule_config: dict):
        self.schedule_config = schedule_config

    def schedule_task(self):
        """Planifie une tâche selon la configuration."""
        pass

    def attach_cron_job(self):
        """Attache une tâche au cron system."""
        pass

    def run_scheduled_task(self):
        """Exécute la tâche planifiée."""
        pass



from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend

class CryptoManager:
    """
    Gestion du chiffrement AES-256 et de la rotation des clés.
    """
    def __init__(self, key: bytes):
        self.key = key

    def encrypt(self, data: bytes) -> bytes:
        """Chiffre les données avec AES-256 (exemple simplifié)."""
        cipher = Cipher(algorithms.AES(self.key), modes.ECB(), backend=default_backend())
        encryptor = cipher.encryptor()
        return encryptor.update(data) + encryptor.finalize()

    def decrypt(self, encrypted_data: bytes) -> bytes:
        """Déchiffre les données avec AES-256."""
        cipher = Cipher(algorithms.AES(self.key), modes.ECB(), backend=default_backend())
        decryptor = cipher.decryptor()
        return decryptor.update(encrypted_data) + decryptor.finalize()

    def rotate_key(self, new_key: bytes):
        """Effectue la rotation de la clé."""
        self.key = new_key



from ldap3 import Server, Connection, ALL, SUBTREE
from ldap3.core.exceptions import LDAPException
from typing import List, Dict, Optional
import ssl

class ADUserManager:
    """Classe pour gérer les interactions LDAP avec Active Directory"""
    
    def __init__(self, server: str, domain: str, username: str, password: str):
        self.server = server
        self.domain = domain
        self.username = username
        self.password = password
        self.conn = None
        self.domain_dn = self._domain_to_dn(domain)
        
    def __enter__(self):
        self.connect()
        return self
        
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.disconnect()
        
    @staticmethod
    def _domain_to_dn(domain: str) -> str:
        """Convertit un nom de domaine en Distinguished Name (DN)"""
        return ','.join([f'DC={part}' for part in domain.split('.')])
    
    def connect(self, verify_ssl: bool = False) -> bool:
        """Établit une connexion sécurisée avec le serveur LDAP"""
        try:
            tls_config = ssl.create_default_context()
            tls_config.check_hostname = verify_ssl
            tls_config.verify_mode = ssl.CERT_REQUIRED if verify_ssl else ssl.CERT_NONE

            ldap_server = Server(
                self.server, 
                use_ssl=True, 
                tls=tls_config
            )
            
            self.conn = Connection(
                ldap_server,
                user=f'{self.username}@{self.domain}',
                password=self.password,
                authentication='NTLM',
                auto_bind=True
            )
            return True
        except LDAPException as e:
            raise ADConnectionError(f"Échec de la connexion LDAP: {e}")
    
    def disconnect(self):
        """Ferme proprement la connexion LDAP"""
        if self.conn:
            self.conn.unbind()
            self.conn = None
            
    def get_all_users(self, attributes: List[str] = None) -> List[Dict]:
        """Récupère tous les utilisateurs du domaine"""
        default_attrs = ['sAMAccountName', 'givenName', 'sn', 'mail', 'userPrincipalName']
        search_filter = "(&(objectClass=user)(objectCategory=person))"
        return self._search_directory(search_filter, attributes or default_attrs)
    
    def get_group_members(self, group_name: str, ou: str = "Groups", recursive: bool = True) -> List[Dict]:
        """Récupère les membres d'un groupe avec gestion de l'imbrication"""
        group_dn = f"CN={group_name},OU={ou},{self.domain_dn}"
        matching_rule = '1.2.840.113556.1.4.1941' if recursive else None
        search_filter = f"(memberOf:{matching_rule if recursive else ''}={group_dn})"
        return self._search_directory(search_filter, ['sAMAccountName', 'memberOf'])
    
    def _search_directory(self, search_filter: str, attributes: List[str]) -> List[Dict]:
        """Exécute une recherche générique dans l'annuaire"""
        if not self.conn or not self.conn.bound:
            raise ADConnectionError("Non connecté au serveur LDAP")
            
        try:
            entries = []
            self.conn.search(
                search_base=self.domain_dn,
                search_filter=search_filter,
                search_scope=SUBTREE,
                attributes=attributes,
                paged_size=1000,
                generator=False
            )
            
            for entry in self.conn.entries:
                entries.append({attr: entry[attr].value for attr in attributes})
                
            return entries
        except LDAPException as e:
            raise ADOperationError(f"Erreur de recherche LDAP: {e}")

class ADConnectionError(Exception):
    """Erreur personnalisée pour les problèmes de connexion"""
    pass

class ADOperationError(Exception):
    """Erreur personnalisée pour les opérations LDAP"""
    pass

# Exemple d'utilisation
if __name__ == "__main__":
    try:
        with ADUserManager(
            server="dc.example.com",
            domain="example.com",
            username="admin_user",
            password="secure_password"
        ) as ad_manager:
            
            # Récupération de tous les utilisateurs
            all_users = ad_manager.get_all_users()
            print(f"Total utilisateurs: {len(all_users)}")
            
            # Récupération des membres d'un groupe
            dev_group = ad_manager.get_group_members("Developers", ou="IT_Groups")
            print(f"Membres du groupe Developers: {len(dev_group)}")
            
    except ADConnectionError as e:
        print(f"Erreur de connexion: {e}")
    except ADOperationError as e:
        print(f"Erreur d'opération: {e}")


import logging
import os
import platform
import subprocess
import sys
import click
import re
from datetime import datetime



@click.group()
def cron():
    """Commandes pour gérer les tâches planifiées."""
    pass

def get_executable_name():
    """Retourne le chemin absolu de l'exécutable"""
    return os.path.abspath(sys.argv[0])

def get_agent_log_path():
    """Retourne le chemin absolu des logs de l'application"""
    log_path = os.path.abspath("watchman_agent_v2/logs/logs.log")
    os.makedirs(os.path.dirname(log_path), exist_ok=True)
    return log_path

def register_system_cron(hour, minute, day, month, command):
    """Enregistre la tâche dans le système de planification"""
    system = platform.system()
    job_name = "WatchmanAgent"
    agent_log = get_agent_log_path()
    
    # Validation des paramètres
    if not re.match(r'^\d{1,2}$', hour) or int(hour) > 23:
        raise ValueError("Heure invalide (0-23)")
    if not re.match(r'^\d{1,2}$', minute) or int(minute) > 59:
        raise ValueError("Minute invalide (0-59)")
    
    try:
        if system in ['Linux', 'Darwin']:
            # Formatage de l'expression cron avec redirection des logs
            cron_expr = f"{minute} {hour} {day} {month} * {command} >> {agent_log} 2>&1"
            
            # Ajout à la crontab
            subprocess.run(
                f'(crontab -l | grep -v "# {job_name}"; echo "{cron_expr} # {job_name}") | crontab -',
                shell=True,
                check=True
            )
            return f"Tâche cron configurée: {cron_expr}"

        elif system == 'Windows':
            time_str = f"{int(hour):02d}:{int(minute):02d}"
            
            # Commande avec redirection pour Windows
            full_cmd = f'cmd /c "{command} >> \"{agent_log}\" 2>&1"'
            
            # Suppression de l'ancienne tâche
            subprocess.run(
                f'schtasks /Delete /TN "{job_name}" /F',
                shell=True,
                stderr=subprocess.DEVNULL,
                stdout=subprocess.DEVNULL
            )
            
            # Configurer selon le type de planification
            if day == '*' and month == '*':
                cmd = f'schtasks /Create /TN "{job_name}" /TR "{full_cmd}" /SC DAILY /ST {time_str} /F'
            elif day != '*' and month == '*':
                cmd = f'schtasks /Create /TN "{job_name}" /TR "{full_cmd}" /SC MONTHLY /D {day} /ST {time_str} /F'
            else:
                months = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', 'SEP', 'OCT', 'NOV', 'DEC']
                month_name = months[int(month)-1] if month != '*' else '*'
                cmd = f'schtasks /Create /TN "{job_name}" /TR "{full_cmd}" /SC ONCE /SD 01/{month}/{datetime.now().year} /ST {time_str} /F'
            
            subprocess.run(cmd, shell=True, check=True)
            return f"Tâche Windows configurée: {cmd}"

        else:
            raise NotImplementedError("Système non supporté")
    except subprocess.CalledProcessError as e:
        raise RuntimeError(f"Erreur de configuration: {e}")

def setup_cron_logging():
    """Configure la journalisation pour les opérations cron"""
    log_path = os.path.abspath("watchman_agent_v2/logs/cron.log")
    os.makedirs(os.path.dirname(log_path), exist_ok=True)
    
    logging.basicConfig(
        filename=log_path,
        level=logging.INFO,
        format="%(asctime)s - %(levelname)s - %(message)s",
        encoding="utf-8"
    )
    return log_path


def unregister_system_cron():
    """Supprime la tâche du système"""
    system = platform.system()
    job_name = "WatchmanAgent"
    
    try:
        if system in ['Linux', 'Darwin']:
            subprocess.run(
                f'(crontab -l | grep -v "# {job_name}") | crontab -',
                shell=True,
                check=True
            )
            return "Tâche cron supprimée"
            
        elif system == 'Windows':
            subprocess.run(
                f'schtasks /Delete /TN "{job_name}" /F',
                shell=True,
                check=True
            )
            return "Tâche Windows supprimée"
            
    except subprocess.CalledProcessError as e:
        raise RuntimeError(f"Erreur de suppression: {e}")

def check_system_cron():
    """Vérifie si la tâche existe"""
    system = platform.system()
    job_name = "WatchmanAgent"
    
    try:
        if system in ['Linux', 'Darwin']:
            result = subprocess.run(
                'crontab -l',
                shell=True,
                capture_output=True,
                text=True
            )
            return f"# {job_name}" in result.stdout
            
        elif system == 'Windows':
            result = subprocess.run(
                f'schtasks /Query /TN "{job_name}"',
                shell=True,
                stderr=subprocess.DEVNULL,
                stdout=subprocess.DEVNULL
            )
            return result.returncode == 0
            
    except Exception:
        return False


@click.command()
@click.option('--hour', type=str, required=True, help="Heure (0-23)")
@click.option('--minute', type=str, required=True, help="Minute (0-59)")
@click.option('--day', type=str, default="*", help="Jour du mois (1-31), '*' par défaut")
@click.option('--month', type=str, default="*", help="Mois (1-12), '*' par défaut")
@click.option('--mode', default="local", type=click.Choice(['local', 'network']), help="Mode d'exécution")
def start(hour, minute, day, month, mode):
    """Configure la tâche planifiée dans le système"""
    setup_cron_logging()
    
    try:
        executable = get_executable_name()
        
        # Commande de base sans redirection
        base_cmd = f'{executable} run --mode {mode}' if mode != "local" else f'{executable} run'
        
        result = register_system_cron(hour, minute, day, month, base_cmd)
        logging.info(result)
        click.echo(result)
        
    except Exception as e:
        logging.error(f"❌ Erreur: {str(e)}")
        click.echo(f"Erreur: {str(e)}", err=True)


@click.command()
def status():
    """Affiche l'état de la tâche planifiée"""
    try:
        if check_system_cron():
            click.echo("En cours")
        else:
            click.echo("Arrêté")
    except Exception as e:
        click.echo(f"Erreur: {str(e)}", err=True)

@click.command()
@click.option("--lines", default=30, help="Nombre de lignes à afficher")
def logs(lines):
    """Affiche les logs de configuration"""
    log_file = "watchman_agent_v2/logs/logs.log"
    
    if not os.path.exists(log_file):
        click.echo("Aucun log disponible")
        return

    try:
        with open(log_file, "r", encoding="utf-8") as f:
            content = f.read().splitlines()[-lines:]
            click.echo("\n".join(content))
    except Exception as e:
        click.echo(f"Erreur de lecture: {str(e)}", err=True)

@click.command()
def stop():
    """Supprime la tâche planifiée"""
    log_file = setup_cron_logging()
    
    try:
        result = unregister_system_cron()
        logging.info(result)
        click.echo(result)
    except Exception as e:
        logging.error(f"❌ Erreur: {str(e)}")
        click.echo(f"Erreur: {str(e)}", err=True)

cron.add_command(start)
cron.add_command(status)
cron.add_command(logs)
cron.add_command(stop)

if __name__ == '__main__':
    cron()


import platform
import json

from watchman_agent_v2.core.utils.log_manager import LogManager
from watchman_agent_v2.core.protocols.protocol_factory import BaseProtocol


class WMIProtocol(BaseProtocol):
    """
    Classe permettant de récupérer des informations système et les applications installées sur une machine Windows.
    Elle gère les connexions locales (via WMI, uniquement sous Windows) et distantes (via WMI sur Windows ou WinRM depuis Linux).
    """
    def __init__(self):
        super().__init__()
        self.conn = None
        self.logger=LogManager
        self.remote_mode = False  # Flag indiquant une connexion distante
        self.results={'system_info':{},'applications':[]}
    

    def fetch_installed_applications(self, ip: str, username: str = None, password: str = None, port: int = 135) -> None:
        """
        Pour chaque machine récupérée, se connecte en remote et extrait la liste des applications installées.
        Utilise WMI sous Windows ou WinRM sous Linux.
        """
        current_os = platform.system().lower()
        self.logger.info(f"Système d'exécution détecté : {current_os}")
        hostname=ip
        self.remote_user=username
        self.remote_password=password
        self.logger.info(f"Récupération des applications sur : {hostname}")
        apps = []
        system_info = {}
        if current_os == 'windows':
            # Utilisation de WMI pour Windows
            try:
                import wmi
                if ip in ["localhost", "127.0.0.1"]:
                    # Connexion locale (uniquement possible sous Windows)
                    if platform.system() != "Windows":
                        raise Exception("La connexion locale WMI est uniquement supportée sous Windows.")
                    try:
                        c = wmi.WMI(computer=ip)
                        print(f"Connexion locale réussie à {ip}")
                    except Exception as e:
                        print(f"Erreur lors de la connexion locale WMI: {str(e)}")
                        raise
                else:
                    c = wmi.WMI(computer=hostname,
                            user=self.remote_user,
                            password=self.remote_password)
                for product in c.Win32_Product():
                    app = {
                        'name': product.Name,
                        'version': product.Version,
                        'vendor': product.Vendor,
                        'type': 'application'  # À adapter si besoin
                    }
                    apps.append(app)
                for os in c.Win32_OperatingSystem():
                    system_info['hostname'] = os.CSName
                    system_info['ip'] = ip
                    system_info['os'] = os.Caption
                    system_info['version'] = os.Version
                    system_info['architecture'] = os.OSArchitecture

                for nic in c.Win32_NetworkAdapterConfiguration(IPEnabled=True):
                    if nic.MACAddress:
                        system_info['mac'] = nic.MACAddress
                        break
                # add os as apps 
                apps.append({
                        'name':system_info['os'],
                        'version':system_info['version'],
                        "vendor":"*",
                        "type":"os"})
                self.logger.info(f"{len(apps)} application(s) trouvée(s) sur {hostname} via WMI")
            except Exception as e:
                self.logger.error(f"Erreur lors de la connexion WMI sur {hostname}: {e}")
        else:
            # Utilisation de WinRM pour Linux (ou autres OS non Windows)
            try:
                import winrm
                # Construit l'URL WinRM pour la machine distante
                # Le port 5985 est utilisé par défaut pour WinRM en HTTP
                
                # Construit l'URL WinRM pour la machine distante
                winrm_url = f'http://{hostname}:5985/wsman'
                session = winrm.Session(winrm_url, auth=(self.config.decrypt(self.remote_user), self.config.decrypt(self.remote_password)), transport='ntlm')
                self.logger.info(f"Connexion avec succès ")
                # La commande PowerShell pour récupérer les applications installées et infos système
                ps_script = """
                $sysinfo = @{
                    hostname = (Get-WmiObject -Class Win32_ComputerSystem).Name
                    os = (Get-WmiObject -Class Win32_OperatingSystem).Caption
                    version = (Get-WmiObject -Class Win32_OperatingSystem).Version
                    architecture = (Get-WmiObject -Class Win32_OperatingSystem).OSArchitecture
                    mac = (Get-WmiObject -Class Win32_NetworkAdapterConfiguration | Where-Object { $_.MACAddress -and $_.IPEnabled -eq $true } | Select-Object -First 1).MACAddress
                }

                $apps = Get-WmiObject -Class Win32_Product |
                    Select-Object Name, Version, Vendor

                @{
                    system_info = $sysinfo
                    applications = $apps
                } | ConvertTo-Json -Depth 3
                """
                result = session.run_ps(ps_script)
                if result.status_code == 0:
                    # La sortie est en JSON
                    data = json.loads(result.std_out.decode('utf-8'))
                    
                    # Extraire les infos
                    system_info = data.get('system_info', {})
                    system_info.update({'ip':ip})
                    apps_data = data.get('applications', [])

                    # Si un seul objet est retourné, le mettre dans une liste
                    if isinstance(apps_data, dict):
                        apps_data = [apps_data]

                    apps = [{
                        'name': app.get('Name', ''),
                        'version': app.get('Version', ''),
                        'vendor': app.get('Vendor', ''),
                        'type': 'application'
                    } for app in apps_data if app.get('Name')]
                    # add os as apps 
                    apps.append({
                        'name':system_info['os'],
                        'version':system_info['version'],
                        "vendor":"*",
                        "type":"os"})
                    self.logger.info(f"{len(apps)} application(s) trouvée(s) sur {hostname} via WinRM")
                else:
                    err = result.std_err.decode('utf-8')
                    self.logger.error(f"Erreur WinRM sur {hostname}: {err}")
            except Exception as e:
                self.logger.error(f"Erreur lors de la connexion WinRM sur {hostname}: {e}")

            # Ajouter les informations récupérées (applications + system_info)
            self.results['applications'] = apps
            self.results['system_info'].update(system_info)


    def collect_info(self, ip: str, config: dict):
        """
        Récupère les informations système et la liste des applications installées.
        
        Pour une connexion locale, aucune authentification n'est nécessaire.
        Pour une connexion distante, le dictionnaire 'config' doit contenir au minimum 'username' et 'password'.
        """
        if ip in ["localhost", "127.0.0.1"]:
            username = None
            password = None
            port = config.get('port', 135)
        else:
            required_keys = {'username', 'password'}
            missing = required_keys - config.keys()
            if missing:
                return {"error": f"Configuration manquante: {missing}"}
            username = config.get('username')
            password = config.get('password')
            port = config.get('port', 5985)  # port pour WMI ou WinRM (par défaut, WinRM utilise 5985)
        
        self.fetch_installed_applications(ip,username,password,port)
        return {
            "system_info": self.results['system_info'],
            "applications": self.results['applications']
        }


if __name__ == "__main__":
    import getpass

    # ip = input("Adresse IP (entrez 'localhost' pour une connexion locale) : ").strip()
    ip='10.10.15.100'
    if ip in ["localhost", "127.0.0.1"]:
        config = {}
    else:
        # username = input("Nom d'utilisateur : ")
        # password = getpass.getpass("Mot de passe : ")
        USER_DN = ""            # Format (ex: "DOMAINE\\utilisateur")
        PASSWORD = ""
        # Pour WinRM, le port par défaut est généralement 5985 (non sécurisé)
        port_input = 135
        port = int(port_input) if port_input else 135
        config = {"username": USER_DN, "password": PASSWORD, "port": port}

    client = WMIProtocol()
    info = client.collect_info(ip, config)
    print(json.dumps(info, indent=4, ensure_ascii=False))



from setuptools import setup, find_packages

with open("Readme.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
    
setup(
    name="watchman-agent",
    version="3.0.4",
    author="Watchman",
    author_email="support@watchman.bj",
    # description = "Watchman Agent 1.0.0",
    description='Agent pour collecter des informations système et les envoyer à une API centrale.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    packages=find_packages(),
    python_requires='>=3.8',
    # py_modules=[''],
    # install_requires=[
    #     'requests',
    #     'sqlitedict',
    #     'scapy',
    #     'keyring',
    #     'python-crontab',
    #     'environs',
    #     'click',
    #     'sqlitedict',
    #     'paramiko',
    #     'pyyaml',
    #     'schedule',
    #     'pysnmplib',
    #     'semver',
    #     'packaging',
    #     'openpyxl',
    #     # 'pandas',
    #     'getmac',
    #     'platformdirs',
    # ],
    install_requires=[
        "altgraph==0.17.4",
        "anyio==4.9.0",
        "APScheduler==3.11.0",
        "bcrypt==4.3.0",
        "blinker==1.9.0",
        "certifi==2025.1.31",
        "cffi==1.17.1",
        "charset-normalizer==3.4.1",
        "click==8.1.7",
        "colorama==0.4.6",
        "cryptography==44.0.2",
        "Cython==3.0.7",
        "environs==10.0.0",
        "et_xmlfile==2.0.0",
        "getmac==0.9.4",
        "h11==0.14.0",
        "httpcore==1.0.8",
        "httpx==0.28.1",
        "idna==3.10",
        "jaraco.classes==3.4.0",
        "keyring==24.3.0",
        "ldap3==2.9.1",
        "marshmallow==3.26.1",
        "more-itertools==10.6.0",
        "openpyxl==3.1.5",
        "packaging==23.2",
        "paramiko==3.4.0",
        "pefile==2024.8.26",
        "platformdirs==4.1.0",
        "ply==3.11",
        "psutil==7.0.0",
        "puresnmp==2.0.1",
        "pyasn1==0.6.1",
        "pycparser==2.22",
        "pycryptodomex==3.22.0",
        "pyinstaller==6.3.0",
        "pyinstaller-hooks-contrib==2025.2",
        "PyNaCl==1.5.0",
        "pysnmp-pyasn1==1.1.3",
        "pysnmp-pysmi==1.1.12",
        "pysnmplib==5.0.23",
        "pyspnego==0.11.2",
        "python-crontab==3.0.0",
        "python-dateutil==2.9.0.post0",
        "python-dotenv==1.1.0",
        "python-magic==0.4.27",
        "python-magic-bin==0.4.14 ; sys_platform == 'win32'",
        "pywin32-ctypes==0.2.3",
        "pywinrm==0.5.0",
        "PyYAML==6.0.1",
        "requests==2.31.0",
        "requests-toolbelt==1.0.0",
        "requests_ntlm==1.3.0",
        "scapy==2.5.0",
        "schedule==1.2.1",
        "semver==3.0.2",
        "setuptools==69.0.2",
        "six==1.17.0",
        "sniffio==1.3.1",
        "sqlitedict==2.1.0",
        "t61codec==1.0.1",
        "typing_extensions==4.12.2",
        "tzlocal==5.3.1",
        "urllib3==2.3.0",
        "Werkzeug==3.1.3",
        "WMI==1.4.9",
        "x690==1.0.0.post1",
        "xmltodict==0.14.2",
        "zipp==3.21.0",
        "wmi==1.4.9 ; sys_platform == 'win32'"
    ],

    entry_points='''
        [console_scripts]
        watchman-agent=watchman_agent_v2.agent:cli
    '''
)



import json
import os
import re
import magic
from typing import Dict, Optional
import click
import requests
from requests_toolbelt.multipart.encoder import MultipartEncoder
from watchman_agent_v2.core.services.api.api_client import APIClient
from watchman_agent_v2.core.services.api.exceptions import InventoryValidationError
from watchman_agent_v2.core.utils.log_manager import LogManager

class AssetAPIClient(APIClient):
    """
    Client spécialisé pour l'envoi de fichiers d'inventaire avec validation avancée
    """
    API_BASE_URL = 'https://api.watchman.bj/api/v2/'
    API_ENDPOINT = 'agent/webhook_v2/'
    ALLOWED_MIME_TYPES = {
        'text/csv': ['csv'],
        'application/json': ['json'],
        'text/plain': ['json'],
        'application/vnd.ms-excel': ['xls'],
        'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': ['xlsx']
    }

    def __init__(self, credentials: Dict[str, str]):
        full_endpoint = f"{self.API_BASE_URL.rstrip('/')}/{self.API_ENDPOINT.lstrip('/')}"
        super().__init__(full_endpoint, credentials)
        self.mime = magic.Magic(mime=True)

    def send_assets(self, file_path: str) -> bool:
        """
        Envoie un fichier d'inventaire avec validation complète
        
        Args:
            file_path: Chemin vers le fichier à envoyer
            
        Returns:
            bool: True si l'envoi réussit
            
        Raises:
            InventoryValidationError: Si validation échoue
        """
        try:
            self._validate_file(file_path)
            
            with open(file_path, 'rb') as f:
                multipart_data = MultipartEncoder(
                    fields={'file': (os.path.basename(file_path), f, self.mime.from_file(file_path))}
                )
                
                headers = {
                    'Content-Type': multipart_data.content_type,
                    **self.credentials
                }
                
                response = self.send_data(
                    data=multipart_data,
                    headers=headers
                )
                
                
                
                error_message=None
                response_data=response.json()
                report=response_data.get('report', {})
                code=response.status_code
                if code !=201 :
                    try:
                        match = re.search(r"'message': ErrorDetail\(string='([^']*)'", response_data['detail'])
                        if match:
                            error_message = match.group(1)
                        else:
                            error_message=response_data['detail']
                    except :
                        error_message = "Serveur Error! Please contact Watchman developper"  # Récupération du message

            return response.status_code == 201,report,error_message

        except FileNotFoundError:
            click.echo(f"Fichier introuvable : {file_path}", err=True)
            raise
        except PermissionError:
            click.echo(f"Permission refusée pour : {file_path}", err=True)
            raise

    def _validate_file(self, file_path: str):
        """Valide le fichier selon les critères de sécurité"""
        # Vérification basique
        if not os.path.isfile(file_path):
            raise InventoryValidationError("Le chemin spécifié n'est pas un fichier valide")
            
        # Taille maximale 100MB
        max_size = 100 * 1024 * 1024
        file_size = os.path.getsize(file_path)
        if file_size > max_size:
            raise InventoryValidationError(f"Fichier trop volumineux ({file_size} > {max_size} bytes)")
            
        # Type MIME réel
        mime_type = self.mime.from_file(file_path)
        extension = os.path.splitext(file_path)[1].lower().lstrip('.')
        
        if mime_type not in self.ALLOWED_MIME_TYPES:
            raise InventoryValidationError(f"Type MIME non autorisé : {mime_type}")
            
        if extension not in self.ALLOWED_MIME_TYPES[mime_type]:
            raise InventoryValidationError(f"Extension incohérente avec le type MIME : {extension} != {self.ALLOWED_MIME_TYPES[mime_type]}")

    def _calculate_checksum(self, file_path: str) -> str:
        """Calcule une empreinte sécurisée du fichier"""
        # Implémentation SHA-256
        import hashlib
        sha256 = hashlib.sha256()
        with open(file_path, 'rb') as f:
            while chunk := f.read(8192):
                sha256.update(chunk)
        return sha256.hexdigest()

    def send_data(self, data, headers: Optional[Dict] = None):
        """Override pour gestion des fichiers"""
        try:
            return self.session.post(
                self.endpoint,
                data=data,
                headers=headers or {},
                timeout=30
            )
        except requests.exceptions.RequestException as e:
            click.echo(f"Erreur réseau : {str(e)}", err=True)
            raise


import platform
import csv
import json
import subprocess
import socket
import uuid
import sys
from typing import List, Dict, Optional, Tuple
import psutil

from watchman_agent_v2.core.utils.log_manager import LogManager

class ManageLocalhost:
    def __init__(self):
        self.os_name, self.os_version = self._get_os_info()
        self.architecture = platform.machine()
        self.hostname = socket.gethostname()
        self.ip_address = self._get_ip_address()
        self.mac_address = self._get_mac_address()
        self.applications = self.get_installed_apps()
        self.virtualization_info = self._detect_virtualization()
        
    def _get_os_info(self) -> Tuple[str, str]:
        """Récupère les informations détaillées du système d'exploitation"""
        system = platform.system()
        if system == 'Darwin':
            version = subprocess.getoutput('sw_vers -productVersion')
            return 'macOS', version
        elif system == 'Linux':
            try:
                with open('/etc/os-release') as f:
                    content = f.read().splitlines()
                    os_info = {k:v.strip('"') for k,v in (line.split('=',1) for line in content if '=' in line)}
                    return os_info.get('PRETTY_NAME', 'Linux'), os_info.get('VERSION_ID', platform.release())
            except:
                return 'Linux', platform.release()
        elif system == 'Windows':
            return 'Windows', platform.win32_ver()[0]
        return 'Unknown', ''

    def _get_ip_address(self) -> str:
        """Récupère l'adresse IP principale"""
        try:
            return socket.gethostbyname(socket.gethostname())
        except:
            return '127.0.0.1'

    def _get_mac_address(self) -> str:
        """Récupère l'adresse MAC principale"""
        try:
            mac_num = hex(uuid.getnode()).replace('0x', '').upper()
            mac = '-'.join(mac_num[i:i+2] for i in range(0,11,2))
            return mac
        except:
            return '00:00:00:00:00:00'

    def _detect_virtualization(self) -> Dict:
        """Détecte l'environnement de virtualisation"""
        virt_info = {
            'is_virtual': False,
            'host_machine': {}
        }

        try:
            if platform.system() == 'Linux':
                virt = subprocess.check_output(['systemd-detect-virt'], text=True).strip()
                if virt != 'none':
                    virt_info['is_virtual'] = True
                    virt_info['host_machine']['type'] = virt
            elif platform.system() == 'Windows':
                import winreg
                with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r'SYSTEM\CurrentControlSet\Control\\') as key:
                    try:
                        virt_type = winreg.QueryValueEx(key, 'SystemManufacturer')[0]
                        if any(x in virt_type for x in ['VMware', 'VirtualBox', 'QEMU']):
                            virt_info['is_virtual'] = True
                            virt_info['host_machine']['type'] = virt_type
                    except:
                        pass
            elif platform.system() == 'Darwin':
                output = subprocess.check_output(['sysctl', '-n', 'machdep.cpu.brand_string'], text=True).strip()
                if 'Virtual' in output:
                    virt_info['is_virtual'] = True
                    virt_info['host_machine']['type'] = output
        except:
            pass

        return virt_info

    def _get_windows_apps(self) -> List[Dict]:
        """Récupère les applications Windows avec les nouvelles métadonnées"""
        import winreg

        apps = []
        registry_keys = [
            (winreg.HKEY_LOCAL_MACHINE, r'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall'),
            (winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Uninstall')
        ]

        for root, key_path in registry_keys:
            try:
                with winreg.OpenKey(root, key_path) as key:
                    for i in range(winreg.QueryInfoKey(key)[0]):
                        subkey_name = winreg.EnumKey(key, i)
                        with winreg.OpenKey(key, subkey_name) as subkey:
                            try:
                                app_data = {
                                    'name': winreg.QueryValueEx(subkey, 'DisplayName')[0],
                                    'version': winreg.QueryValueEx(subkey, 'DisplayVersion')[0],
                                    'vendor': winreg.QueryValueEx(subkey, 'Publisher')[0],
                                    'type': 'application'
                                }
                                if app_data['name']:
                                    apps.append(app_data)
                            except OSError:
                                continue
            except OSError:
                continue

        return apps

    def _get_linux_apps(self) -> List[Dict]:
        """Récupère les applications Linux avec les nouvelles métadonnées"""
        apps = []

        # Détection des paquets Debian/Ubuntu
        try:
            output = subprocess.check_output(
                ['dpkg-query', '-W', '-f=${Package};${Version};${Maintainer}\n'],
                text=True
            )
            for line in output.split('\n'):
                if line.strip():
                    name, version, vendor = line.strip().split(';', 2)
                    apps.append({
                        'name': name,
                        'version': version,
                        'vendor': vendor,
                        'type': 'application'
                    })
            return apps
        except:
            pass

        # Détection des paquets RPM
        try:
            output = subprocess.check_output(
                ['rpm', '-qa', '--queryformat', '%{NAME};%{VERSION};%{VENDOR}\n'],
                text=True
            )
            for line in output.split('\n'):
                if line.strip():
                    name, version, vendor = line.strip().split(';', 2)
                    apps.append({
                        'name': name,
                        'version': version,
                        'vendor': vendor,
                        'type': 'application'
                    })
            return apps
        except:
            pass

        return apps

    def _get_macos_apps(self) -> List[Dict]:
        """Récupère les applications macOS avec les nouvelles métadonnées"""
        apps = []
        try:
            output = subprocess.check_output(['system_profiler', 'SPApplicationsDataType', '-xml'], text=True)
            import plistlib
            plist = plistlib.loads(output.encode('utf-8'))
            for app in plist[0]['_items']:
                apps.append({
                    'name': app.get('_name', ''),
                    'version': app.get('version', ''),
                    'vendor': app.get('obtained_from', ''),
                    'type': 'application'
                })
        except Exception as e:
            LogManager.error(f"Erreur applications macOS: {e}", file=sys.stderr)
        
        return apps

    def get_installed_apps(self) -> List[Dict]:
        """Récupère les applications avec métadonnées étendues"""
        apps = []

        # Ajout de l'OS comme première application
        apps.append({
            'name': self.os_name,
            'version': self.os_version,
            'vendor': self.os_name,
            'type': 'os'
        })

        # Récupération des autres applications
        if platform.system() == 'Windows':
            apps += self._get_windows_apps()
        elif platform.system() == 'Darwin':
            apps += self._get_macos_apps()
        else:
            apps += self._get_linux_apps()

        return apps

    def export_to_csv(self, filename: str = 'applications.csv'):
        """Exporte les données au format CSV professionnel"""
        try:
            with open(filename, 'w', newline='', encoding='utf-8') as csvfile:
                writer = csv.writer(csvfile, quoting=csv.QUOTE_MINIMAL)
                
                # Entête selon les spécifications
                headers = [
                    'ip', 'mac', 'architecture', 'hostname', 'os',
                    'stack_name', 'stack_version', 'stack_type', 'stack_vendor',
                    'host_machine', 'host_machine_architecture',
                    'host_machine_os', 'host_machine_hostname', 'host_machine_mac'
                ]
                writer.writerow(headers)

                for app in self.applications:
                    row = [
                        self.ip_address,
                        self.mac_address,
                        self.architecture,
                        self.hostname,
                        f"{self.os_name} {self.os_version}",
                        app['name'],
                        app['version'],
                        app['type'],
                        app['vendor'],
                        self.virtualization_info['host_machine'].get('type', '') if self.virtualization_info['is_virtual'] else '',
                        '',  # host_machine_architecture
                        '',  # host_machine_os
                        '',  # host_machine_hostname
                        ''   # host_machine_mac
                    ]
                    writer.writerow(row)
            
            LogManager.success(f"CSV généré avec succès : {filename}")
        except Exception as e:
            LogManager.error(f"Erreur CSV: {str(e)}", file=sys.stderr)

    def export_to_json(self, filename: str = 'system_info.json'):
        """Exporte les données au format JSON professionnel"""
        data = {'ip': self.ip_address,
            'mac': self.mac_address,
            'architecture': self.architecture,
            'os': f"{self.os_name} {self.os_version}",
            'hostname': self.hostname,
            'host_machine': "",
            'host_machine_hostname': "",
            'host_machine_os': "",
            'host_machine_architecture': "",
            'host_machine_mac': "",
            'applications': [
                {
                    'name': app['name'],
                    'version': app['version'],
                    'vendor': app['vendor'],
                    'type': app['type']
                }
                for app in self.applications
            ]}
        
        

        # Ajout des informations de virtualisation si nécessaire
        if self.virtualization_info['is_virtual']:
            data.update({
                'host_machine': self.virtualization_info['host_machine'].get('type', ''),
                'host_machine_hostname': '',
                'host_machine_os': '',
                'host_machine_architecture': '',
                'host_machine_mac': ''
            })
            
        data_array={
            'assets':[
                data
            ]
        }

        try:
            with open(filename, 'w', encoding='utf-8') as jsonfile:
                json.dump(data_array, jsonfile, indent=4, ensure_ascii=False)
            LogManager.success(f"JSON généré avec succès : {filename}")
        except Exception as e:
            LogManager.error(f"Erreur JSON: {str(e)}", file=sys.stderr)

# how to use it 
if __name__ == '__main__':
    manager = ManageLocalhost()
    manager.export_to_csv()
    manager.export_to_json()




import binascii
import re


class ExtractSnmpInfo:

    @staticmethod 
    def extract_info_linux(package_string):
        # Original regular expression pattern
        original_pattern = r'^(.*?)_([^_]+)_([^\s]+)$'

        # Additional pattern to handle strings like "whiptail-0.52.20-1ubuntu1"
        additional_pattern = r'^([^-\s]+)-([\d.]+)-(\S+)$'

        # Pattern for strings like "ncurses-6.1-10.20180224.el8"
        new_pattern = r'^([^-\s]+)-([\d.]+)-([\d.]+)\.([^\s]+)$'

        # Pattern for strings like "ncurses-6.1-10.20180224.el8"
        second_pattern = r'(.*)-(.*)-(.*).(.*)'
        third_pattern = r'\d+(\.\d+)*'

        # Try the original pattern first
        match = re.match(original_pattern, package_string)

        if match:
            name = match.group(1)
            version = match.group(2)
            architecture = match.group(3)
            return {"name": name, "version": version, "arch": architecture}
        else:
            # If the original pattern didn't match, try the additional pattern
            match = re.match(additional_pattern, package_string)

            if match:
                name = match.group(1)
                version = match.group(2)
                architecture = match.group(3)
                return {"name": name, "version": version, "arch": architecture}
            else:
                # If the additional pattern didn't match, try the new pattern
                match = re.match(new_pattern, package_string)

                if match:
                    name = match.group(1)
                    version = match.group(2)
                    release = match.group(3)
                    architecture = match.group(4)
                    return {"name": name, "version": version, "arch": architecture}
                else:
                    # If the new pattern didn't match, try the second pattern
                    match = re.match(second_pattern, package_string)

                    if match:
                        name = match.group(1)
                        version = match.group(2)
                        release = match.group(3)
                        architecture = match.group(4)
                        return {"name": name, "version": version, "arch": architecture}
                    else:
                        return None
    def extract_info_windows(package_string):
        try:
            # Essayer de décoder la chaîne hexadécimale
            decoded_string = binascii.unhexlify(package_string[2:]).decode('latin-1')
        except (binascii.Error, ValueError):
            # Si ce n'est pas une chaîne hexadécimale, utiliser directement la valeur
            decoded_string = package_string

        # Regex améliorée pour capturer des formats plus variés
        pattern = r'^(.*?)\s*-\s*([\d\w\.\-]+)\s*(?:\((.*?)\))?$'
        match = re.match(pattern, decoded_string)

        if match:
            name = match.group(1).strip()
            version = match.group(2).strip()
            architecture = match.group(3).strip() if match.group(3) else None
            return {"name": name, "version": version, "architecture": architecture}

        # Vérifier si la chaîne se termine par un numéro (potentielle version)
        version_match = re.search(r'([\d]+(\.\d+)+)$', decoded_string)
        if version_match:
            name = decoded_string[:version_match.start()].strip()
            version = version_match.group(1)
            return {"name": name, "version": version, "architecture": None}

        # Retourner au cas où aucun pattern ne correspond
        return {"name": decoded_string.strip(), "version": None, "architecture": None}
    @staticmethod
    def extract_windows_host_info(input_str):
        # Expression régulière pour extraire l'architecture matérielle (premier mot après "Hardware:")
        hardware_architecture_pattern = r'Hardware:\s(\S+)'

        # Expression régulière pour extraire les informations logicielles
        software_pattern = r'Software:\s(.*?)\sVersion\s(\d+\.\d+).*\(Build\s(\d+)\s.*\)$'

        # Rechercher les correspondances dans la chaîne d'entrée
        hardware_architecture_match = re.search(hardware_architecture_pattern, input_str, re.IGNORECASE)
        software_match = re.search(software_pattern, input_str)

        # Extraire l'architecture matérielle
        hardware_architecture = hardware_architecture_match.group(1).strip() if hardware_architecture_match else None

        # Extraire les informations logicielles
        software_name = software_match.group(1).strip() if software_match else None
        software_version = software_match.group(2).strip() if software_match else None
        software_build = software_match.group(3).strip() if software_match else None

        return {
            "arch": hardware_architecture,
            "os_name": software_name,
            "kernel_version": software_version,
            "build": software_build
        }

    @staticmethod
    def extract_info_macos(package_string):
        # Utilisation d'une expression régulière pour extraire le nom et la version sous macOS
        pattern = r'^(.*?)\s+(\S+)\s*$'
        match = re.match(pattern, package_string)

        if match:
            name = match.group(1)
            version = match.group(2)
            if '(null)' in version:
                version = None
            if '(null)' in name:
                name = None
            return {"name": name, "version": version, "arch": None}
        else:
            return None



import os
import yaml
import base64
from cryptography.fernet import Fernet
from dotenv import load_dotenv

class CryptManager:
    def __init__(self):
        self.crypto_key = self._load_crypto_key()

    def _load_crypto_key(self):
        """Charge la clé de chiffrement depuis .env"""
        load_dotenv()
        key = os.getenv("CRYPTO_KEY")
        if not key:
            raise ValueError("CRYPTO_KEY est manquant dans .env !")
        return key.encode()

    def encrypt(self, plain_text):
        """Chiffre une donnée sensible."""
        fernet = Fernet(self.crypto_key)
        encrypted_value = base64.urlsafe_b64encode(fernet.encrypt(plain_text.encode())).decode()
        return encrypted_value

    def decrypt(self, encrypted_value):
        """Déchiffre une donnée sensible."""
        if not encrypted_value:
            return None
        fernet = Fernet(self.crypto_key)
        return fernet.decrypt(base64.urlsafe_b64decode(encrypted_value)).decode()



import os
import subprocess
import platform
import click


def launch_detached(cmd, task_name, log_file_path):
    """
    Lance une commande en arrière-plan, enregistre le PID et redirige les logs.
    Chaque tâche est identifiée par un nom (task_name), avec son propre fichier PID.
    """

    pid_dir = "watchman_agent_v2/pids/"
    directory = os.path.dirname(pid_dir)
    os.makedirs(directory, exist_ok=True)
    pid_file_path = os.path.join(pid_dir, f"{task_name}.pid")

    def save_pid(pid):
        with open(pid_file_path, "w") as f:
            f.write(str(pid))

    def is_process_running(pid):
        try:
            os.kill(pid, 0)
        except OSError:
            return False
        else:
            return True

    def check_existing_pid():
        if os.path.exists(pid_file_path):
            try:
                with open(pid_file_path, "r") as f:
                    pid = int(f.read().strip())
                if is_process_running(pid):
                    click.echo(f"⚠️ Une tâche '{task_name}' est déjà en cours (PID {pid})")
                    return True
            except Exception:
                pass
        return False

    if check_existing_pid():
        click.confirm("Souhaitez-vous écraser l'ancien processus ?", abort=True)

    with open(log_file_path, "w") as f:
        kwargs = {
            "stdout": f,
            "stderr": subprocess.STDOUT,
            "close_fds": True
        }
        if platform.system() != "Windows":
            kwargs["start_new_session"] = True

        p = subprocess.Popen(cmd, **kwargs)
        save_pid(p.pid)
        click.echo(f"✅ Processus '{task_name}' lancé (PID {p.pid}, logs: {log_file_path})")






import json
import unittest
from unittest import mock
from unittest.mock import patch, MagicMock

import paramiko
from paramiko.client import SSHClient
from pysnmp.entity.engine import SnmpEngine

from watchman_agent_v2.core.agents.local_agent import LocalAgent
from watchman_agent_v2.core.protocols.http import HttpProtocol, ProtocolError
import httpx

from watchman_agent_v2.core.protocols.ssh import SSHProtocol
from watchman_agent_v2.core.protocols.wmi_protocol import WMIProtocol


class TestLocalAgent(unittest.TestCase):
    def test_run(self):
        agent = LocalAgent()
        # Tester la méthode run (ici, on attend None car pass)
        self.assertIsNone(agent.run())


class TestHttpProtocolLive(unittest.TestCase):

    def test_collect_info_live(self):
        """Test réel qui récupère les données d'un serveur en cours d'exécution"""
        config = {
            'api_key': 'a1ee56ca-adb3-4223-ab77-7a914af8a7c2',
            'port': 9001,
            'endpoint': '/apps',
            'timeout': 30
        }
        ip = '127.0.0.1'

        http_protocol = HttpProtocol()

        try:
            # Récupération des infos réelles
            result = http_protocol.collect_info(ip, config)

            # Affichage du résultat JSON
            print(json.dumps(result, indent=4))

            # Vérification basique de la réponse
            self.assertIsInstance(result, dict)  # Doit retourner un dictionnaire

        except ProtocolError as e:
            self.fail(f"Erreur lors de la récupération des données : {str(e)}")


class TestSNMPProtocol(unittest.TestCase):

    # Test pour la méthode connect
    def test_connect(self):
        protocol = SNMPProtocol(community="public", ip="127.0.0.1")

        # On patch le moteur SNMP pour simuler une connexion
        with mock.patch.object(SnmpEngine, 'start') as mock_start:
            protocol.connect()
            mock_start.assert_called_once()  # Vérifie que l'engine SNMP a été initialisé
            self.assertIsNotNone(protocol.snmp_engine)  # Vérifie que l'engine SNMP a bien été initialisé
            print("SNMP engine bien initialisé")

    # Test pour collect_info quand tout se passe bien avec une réponse correcte
    def test_collect_info_success(self):
        """Test un appel réussi pour collecter des informations"""
        fake_response = mock.Mock()
        fake_response.status_code = 200
        fake_response.json.return_value = {"data": "success"}
        self.protocol.client.get.return_value = fake_response

        ip = "127.0.0.1"
        config = {'api_key': 'fake_api_key', 'port': 9000}

        result = self.protocol.collect_info(ip, config)

        self.assertEqual(result, {"data": "success"})  # Vérifie que la réponse est correctement formatée

    def test_connect(self):
        """Test que la méthode connect initialise bien le client"""
        with mock.patch.object(httpx, 'Client') as MockClient:
            mock_client = mock.Mock()
            MockClient.return_value = mock_client

            self.protocol.connect()
            MockClient.assert_called_once()  # Vérifie que Client a été initialisé

    def test_disconnect(self):
        """Test que la méthode disconnect ferme correctement la connexion"""
        self.protocol.client = mock.Mock()

        with mock.patch.object(self.protocol.client, 'close') as mock_close:
            self.protocol.disconnect()
            mock_close.assert_called_once()  # Vérifie que la méthode 'close' a été appelée une fois


class TestSSHProtocol(unittest.TestCase):

    def test_collect_info_success(self):
        config = {
            'username': 'Utilisateur',  # Remplacez par votre nom d'utilisateur SSH
            'password': '1622001',  # Remplacez par votre mot de passe SSH
            'port': 22  # Port SSH (par défaut, c'est 22)
        }
        ip = '127.0.0.1'  # Remplacez par l'IP de votre serveur

        ssh_protocol = SSHProtocol()

        # Récupérer les informations réelles du serveur
        result = ssh_protocol.collect_info(ip, config)

        # Afficher le résultat JSON
        print(json.dumps(result, indent=4))


class TestWMIProtocol(unittest.TestCase):

    def test_collect_info_success(self):
        config = {
            'username': 'utilisateur',  # Remplacez par votre nom d'utilisateur SSH
            'password': '1622001',  # Remplacez par votre mot de passe SSH
            'port': 135
        }
        ip = '127.0.0.1'  # Remplacez par l'IP de votre serveur

        wmi_protocol = WMIProtocol()

        # Récupérer les informations réelles du serveur
        result = wmi_protocol.collect_info(ip, config)

        # Afficher le résultat JSON
        print(json.dumps(result, indent=4))


# class TestSNMPProtocol(unittest.TestCase):
#
#     # Test pour la méthode connect
#     def test_connect(self):
#         protocol = SNMPProtocol(community="public", ip="127.0.0.1")
#
#         # On patch le moteur SNMP pour simuler une connexion
#         with mock.patch.object(SnmpEngine, 'start') as mock_start:
#             protocol.connect()
#             mock_start.assert_called_once()  # Vérifie que l'engine SNMP a été initialisé
#             self.assertIsNotNone(protocol.snmp_engine)  # Vérifie que l'engine SNMP a bien été initialisé
#             print("SNMP engine bien initialisé")
#
#     # Test pour collect_info quand tout se passe bien avec une réponse correcte
#     def test_collect_info_success(self):
#         protocol = SNMPProtocol(community="public", ip="127.0.0.1")
#         protocol.snmp_engine = SnmpEngine()  # Simuler l'initialisation de l'engine
#
#         # Simuler la réponse d'une requête SNMP avec un OID simple
#         with mock.patch('your_module.getCmd',
#                         return_value=(None, None, None, [(ObjectIdentity('SNMPv2-MIB', 'sysName', 0), 'TestDevice')])):
#             data = protocol.collect_info()
#             self.assertIsNotNone(data)  # Vérifie que les données ont été récupérées
#             self.assertEqual(data['SNMPv2-MIB::sysName.0'],
#                              'TestDevice')  # Vérifie que les données récupérées sont correctes
#             print("Données récupérées :", data)
#
#     # Test pour collect_info avec une erreur SNMP
#     def test_collect_info_snmp_error(self):
#         protocol = SNMPProtocol(community="public", ip="127.0.0.1")
#         protocol.snmp_engine = SnmpEngine()  # Simuler l'initialisation de l'engine
#
#         # Simuler une erreur de SNMP avec un retour d'erreur dans le getCmd
#         with mock.patch('your_module.getCmd', side_effect=Exception("Erreur SNMP")):
#             result = protocol.collect_info()
#             self.assertIsNone(result)  # Vérifie que la méthode retourne None en cas d'erreur SNMP
#
#     # Test pour collect_info avec une erreur de réponse vide ou incorrecte
#     def test_collect_info_empty_response(self):
#         protocol = SNMPProtocol(community="public", ip="127.0.0.1")
#         protocol.snmp_engine = SnmpEngine()  # Simuler l'initialisation de l'engine
#
#         # Simuler une réponse vide de SNMP (aucune donnée retournée)
#         with mock.patch('your_module.getCmd', return_value=(None, None, None, [])):
#             result = protocol.collect_info()
#             self.assertIsNone(result)  # Vérifie que la méthode retourne None si la réponse est vide ou incorrecte
#
#     # Test pour disconnect
#     def test_disconnect(self):
#         protocol = SNMPProtocol(community="public", ip="127.0.0.1")
#         protocol.snmp_engine = SnmpEngine()  # Simuler l'initialisation de l'engine
#
#         # Simuler la déconnexion
#         with mock.patch.object(protocol.snmp_engine, 'cleanup') as mock_cleanup:
#             protocol.disconnect()
#             mock_cleanup.assert_called_once()  # Vérifie que la méthode cleanup a été appelée
#             self.assertIsNone(protocol.snmp_engine)  # Vérifie que l'engine SNMP a été nettoyé
#             print("SNMP engine bien nettoyé après déconnexion")


if __name__ == '__main__':
    unittest.main()



import os
import time
import logging
from typing import Dict, Optional
from threading import Lock
import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from watchman_agent_v2.core.services.api.exceptions import AuthenticationError

# Configuration du logger
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

class AuthClient:
    """
    Client d'authentification sécurisé avec gestion de token JWT/OAuth2
    
    Features :
    - Gestion du cycle de vie du token
    - Rafraîchissement automatique
    - Stockage sécurisé en mémoire
    - Thread-safe
    - Gestion des erreurs
    """
    AUTH_ENDPOINT = "https://api.watchman.bj/api/v2/agent/connect/"
    
    def __init__(
        self,
        client_id: str,
        client_secret: str,
        token_ttl: int = 3600,
        scope: str = "read write"
    ):
        self.client_id = client_id
        self.client_secret = client_secret
        self.auth_endpoint = self.AUTH_ENDPOINT
        self.scope = scope
        self.token_ttl = token_ttl
        self._token: Optional[str] = None
        self._expires_at: float = 0.0
        self._lock = Lock()
        self.session = self._configure_session()

    def _configure_session(self) -> requests.Session:
        """Configure une session HTTP sécurisée"""
        session = requests.Session()
        
        # Configuration TLS
        # session.verify = True
        # session.cert = os.getenv('SSL_CERT_PATH', None)
        
        # Politique de retry
        retry = Retry(
            total=3,
            backoff_factor=0.5,
            status_forcelist=[500, 502, 503, 504],
            allowed_methods=['POST']
        )
        adapter = HTTPAdapter(max_retries=retry)
        session.mount('http://', adapter)
        
        return session

    def _request_token(self) -> Dict:
        """Demande un nouveau token d'accès"""
        try:
            
            data={
                    'client_id': self.client_id,
                    'client_secret': self.client_secret,
                    'grant_type': 'client_credentials',
                    'scope': self.scope
                }
            response = self.session.get(
                self.auth_endpoint,
                headers={
                    'Content-Type': 'application/json',
                    "AGENT-ID": self.client_id,
                    "AGENT-SECRET": self.client_secret
                },
                timeout=10
            )
            response.raise_for_status()
            
            
            return response.json()
            
        except requests.exceptions.RequestException as e:
            logger.error(f"Erreur d'authentification: {str(e)}")
            raise AuthenticationError("Échec de l'authentification") from e

    def _is_token_valid(self) -> bool:
        """Vérifie si le token est toujours valide"""
        return time.time() < self._expires_at - 60  # 60s de marge

    def get_token(self) -> str:
        """Récupère le token valide (thread-safe)"""
        with self._lock:
            if not self._is_token_valid():
                self._refresh_token()
            return self._token

    def _refresh_token(self):
        """Rafraîchit le token d'accès"""
        try:
            token_data = self._request_token()
            self._token = token_data['token']
            # self._expires_at = time.time() + token_data.get('expires_in', self.token_ttl)
            
            # Sécurité : effacer les données sensibles
            del token_data
            
        except KeyError as e:
            logger.error("Réponse d'authentification invalide")
            raise AuthenticationError("Réponse API incorrecte") from e

    def revoke_token(self):
        """Révoque le token actuel (implémentation de base)"""
        with self._lock:
            self._token = None
            self._expires_at = 0.0
            # Implémenter l'appel à l'endpoint de révocation ici

    def test_connection(self) -> bool:
        """Teste la connectivité au service d'authentification"""
        try:
            response = self.session.head(
                self.auth_endpoint,
                timeout=5
            )
            print(self.auth_endpoint)
            return response.status_code in [200, 405,412]
        except requests.exceptions.RequestException as error:
            return False




import os

import click

from watchman_agent_v2.core.agents.local_agent import LocalAgent
from watchman_agent_v2.core.agents.network_agent import NetworkAgent
from watchman_agent_v2.core.utils.launch_detached import launch_detached
from watchman_agent_v2.core.config.config_manager import ConfigManager


@click.command()
@click.option('--mode', type=click.Choice(['local', 'network']), help="Mode d'exécution de l'agent", required=False)
@click.option('-d', '--detach', is_flag=True, help="Exécution de l'agent en arrière plan")
def run(mode, detach):
    """
    Commande pour lancer l'agent en mode local ou réseau.
    """
    if detach:
        pid_file = os.path.expanduser("~/.watchman_agent.pid")
        log_file = os.path.expanduser("~/watchman_agent_v2.log")
        # Construire la commande à lancer en arrière-plan.
        # On ne passe pas l'option -d au subprocess pour éviter une récursion.
        cmd = ["watchman-agent", "run"]
        if mode:
            cmd.extend(["--mode", mode])
        else:
            # Si le mode n'est pas passé, on peut le récupérer depuis la config ou utiliser 'local' par défaut.
            cmd.extend(["--mode", "local"])
        # with open(log_file, "w") as f:
        #     subprocess.Popen(
        #         cmd,
        #         stdout=f, stderr=subprocess.STDOUT,
        #         close_fds=True,
        #         start_new_session=True  # Détache complètement le processus
        #     )
        # click.echo(f"✅ Processus lancé en arrière-plan (logs: {log_file})")

        launch_detached(cmd, "agent", log_file)
        return  # On quitte pour ne pas exécuter la suite dans ce processus

    if mode is None:
        configManager = ConfigManager()
        config = configManager.config
        mode = config.get('mode', 'local')
        if mode is None:
            mode = 'local'

    if mode == "local":
        agent = LocalAgent()
    else:
        agent = NetworkAgent()
    agent.run()


if __name__ == '__main__':
    run()
